unit Form_Client;
{Historic:         https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xml

Revision 1 of the 24/03/2018  ("V.1"), mainly management of the RX/TX interface:
* SwitchTX, SwitchRX, ADJUrate and TXHWdata procedures
* new HARDWARE_TYPE = 5 (XCVR) considered
* transmission of simulated data
* small modifications to be homogeneous with the server program

Revision 0.1 of the 20/03/2018  ("V.0.1"), mainly
* addition of the lenght of the command before the command (redundant information which can simplify reception)
* modification of the default TCP/IP port (50001 instead 3020)
This client can be interfaced with the simulation of a server SDR TCP/IP INTERFACE VERSION 0.1

Revision 0 of the 24/02/2017:
 * original issue for a client SDR TCP/IP INTERFACE VERSION 0 (not applicable for higher versions)
   issued directly from MULTIPSK V.4.31.4 (several procedures are in French).
   This client manages only the mandatory part of the version 0 of the protocol, i.e.:
    * the defaut sampling frequency (48000 Hz),
    * the default type managed: 3 (16 bits, IQ),
    * none MP GUI
    * the mandatory procedures for a CP (Pversion (RX/TX) HWMPcapa (RX/TX)  InitHWMP (TX) TxHWdata (RX) Set_HWLO (RX/TX) Received (TX) Bug_on_P (RX/TX) HW_Pause (RX))
   This client is expected to be interfaced with the simulation of a server SDR TCP/IP INTERFACE VERSION 0
   (which must be started up before the client), so the TCP/IP is fixed and equal to 127.0.0.1 (local loop).
   For a distant server, the IP address might be rendered configurable.

   The number of data received is displayed in the blue label (in Kdata).

   This client manages also the Hardware type 4 (for frequency control only), also managed by MULTIPSK V.4.31.4.
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ScktComp, Buttons, ComCtrls,ShellApi, ExtCtrls;

type
  TForm1 = class(TForm)
    GroupBox3: TGroupBox;
    Memo_Reception_Text: TMemo;
    GroupBox2: TGroupBox;
    GroupBox4: TGroupBox;
    Memo_Log: TMemo;
    Connection: TSpeedButton;
    Disconnection: TSpeedButton;
    Edit1: TEdit;
    Manual_command: TButton;
    Label5: TLabel;
    GroupBox7: TGroupBox;
    Memo_reception_commands: TMemo;
    ClientSocket1: TClientSocket;
    Send_frequency: TButton;
    Timer1: TTimer;
    Label_Client: TLabel;
    Label_server: TLabel;
    Label_bug: TLabel;
    Label_info: TLabel;
    NOMBRE_MILLIERS_RX: TLabel;
    Send_TX_and_data: TSpeedButton;
    NOMBRE_MILLIERS_TX: TLabel;
    procedure ConnectionClick(Sender: TObject);
    procedure DisconnectionClick(Sender: TObject);
    procedure ClientSocket1Connect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocket1Connecting(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocket1Disconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocket1Error(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocket1Read(Sender: TObject; Socket: TCustomWinSocket);
    procedure FormActivate(Sender: TObject);
    procedure Manual_commandClick(Sender: TObject);
    procedure Send_frequencyClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Send_TX_and_dataClick(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}

TYPE TYPE_PARAMETRES=ARRAY[1..255] OF STRING;

VAR Datetime:TDateTime;
var Date_Str: string;

{Variables spcifiques  l'interface SDR TCP/IP INTERFACE}
VAR TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:BOOLEAN;
{The CP client must consider that after 10 seconds waiting for a MP event
(end of transmission of a command), the MP has some problem (often as a freezing)
and the CP will disconnect from the CP (and will return to standard working on the
sound card, for example). Just before the disconnection, it will be transmitted
the procedure Bug_on_P with the parameter "Disconnection due to a too long waiting time"}
VAR CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:BOOLEAN;
CONST Maximum_delay_to_receive_the_expected_event=10000;{ms, so 10 seconds}
VAR Time_waiting:DWORD;{to compare with Maximum_delay_to_receive_the_expected_event}
VAR Initial_time_of_an_event_for_waiting_delay:DWORD;
VAR COMMAND_RECEIVED:STRING;
VAR COMMAND_TO_BE_RECEIVED:STRING;
VAR STRING_RECEIVED_AND_SAVED:STRING;
VAR CURRENT_STRING_RECEIVED:STRING;
VAR MP_VERSION:BYTE; {Particular to Multipsk 4.31.4 which manages only the version 0, in the future will be more than 0}
VAR DISPLAY_INFORMATION_MESSAGE_ON_DISCONNECTION:BOOLEAN;
VAR MANAGEMENT_TCP_IP_RECEPTION_FINISHED:BOOLEAN;{to avoid re-coming}
VAR PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED,NUMBER_OF_CHARACTERS_RECEIVED:DWORD;
VAR HARDWARE_TYPE:BYTE;{3, 4 or 5}
{V.1}
VAR SIMULATED_DATA_TO_SEND:BOOLEAN;
VAR INITIAL_TIME_OF_DATA_TRANSMISSION:DWORD;
VAR NUMBER_OF_IQ_SAMPLES_SENT:DWORD;
VAR PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED,NUMBER_OF_CHARACTERS_TRANSMITTED:DWORD;
VAR RECEPTION_DATA_IN_PROGRESS:BOOLEAN;{for information, not used}
VAR TCP_IP_FLOW_DATA_RATE:LONGINT;{48000 samples/s normally but 49000 initially}

{************}

procedure INITIALIZATION_TCP_IP_EN_RX;
BEGIN
 {Initialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE;
 COMMAND_RECEIVED:='';
 COMMAND_TO_BE_RECEIVED:='Pversion';{the first one to be received after connection and transmission of Pversion}
 CURRENT_STRING_RECEIVED:='';
 STRING_RECEIVED_AND_SAVED:='';//V.1
 Initial_time_of_an_event_for_waiting_delay:=0;
 CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE;
 DISPLAY_INFORMATION_MESSAGE_ON_DISCONNECTION:=FALSE;
 NUMBER_OF_CHARACTERS_RECEIVED:=0;
 PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED:=0;
 PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED:=0;
 NUMBER_OF_CHARACTERS_TRANSMITTED:=0;
 RECEPTION_DATA_IN_PROGRESS:=TRUE;{V.1 RX initially}
END;

{*************}

PROCEDURE REDUCTION_CHAINE(VAR CHAINE:STRING);
{the string is reduced by removing initial and final spaces}
{de CHAINE, on enlve les espaces  premiers et finaux (ou CHR(13) ou CHR(10) inutiles}
VAR TAMPON:STRING;
VAR COMPTEUR:LONGINT;
VAR CARACTERE:CHAR;
VAR POSITION:LONGINT;
VAR NOMBRE:LONGINT;
BEGIN
 {on passe les premiers espaces}
 POSITION:=1; {initialisation}
 FOR COMPTEUR:=1 TO LENGTH(CHAINE) DO
 BEGIN
  CARACTERE:=CHAINE[COMPTEUR];
   IF (CARACTERE=' ') OR (CARACTERE=CHR(13)) OR (CARACTERE=CHR(10)) THEN
    POSITION:=COMPTEUR+1 {on passe} ELSE BREAK;{on a termin}
 END;

 {on note le nombre d'espaces finaux ou de CHR(1O) ou CHR(13) contigus}
 NOMBRE:=0;{initialisation}
 FOR COMPTEUR:=LENGTH(CHAINE) DOWNTO POSITION DO
 BEGIN
  CARACTERE:=CHAINE[COMPTEUR];
  IF (CARACTERE=' ') OR (CARACTERE=CHR(13)) OR (CARACTERE=CHR(10)) THEN INC(NOMBRE)
  ELSE BREAK;{on a termin}
 END;

 {on extrait la chane sans espaces premiers et finaux}
 TAMPON:='';{initialisation}
 FOR COMPTEUR:=POSITION TO LENGTH(CHAINE)-NOMBRE DO
 BEGIN
  CARACTERE:=CHAINE[COMPTEUR];
  TAMPON:=TAMPON+CARACTERE;
 END;
 CHAINE:=TAMPON;
END;


{*************}

PROCEDURE RECHERCHE_POSITION_PREMIER_CARACTERE(CHAINE:STRING;CARACTERE:CHAR;VAR POSITION_DU_CARACTERE:LONGINT);
{Search the first position of a character in a string CHAINE}
{on regarde s'il existe dans une chaine CARACTERE, on donne le
la position du premier (POSITION_CARACTERE), POSITION=0 si chec}
VAR COMPTEUR,LONGUEUR:LONGINT;
VAR CARAC:CHAR;
BEGIN
 {initialisation}
 POSITION_DU_CARACTERE:=0;
 LONGUEUR:=LENGTH(CHAINE);

 {dtection de la position du premier caractre}
 FOR COMPTEUR:=1 TO LONGUEUR DO
 BEGIN
  CARAC:=CHAINE[COMPTEUR];
  IF CARAC=CARACTERE THEN
  BEGIN
   POSITION_DU_CARACTERE:=COMPTEUR;
   BREAK; {on arrte la boucle}
  END;
 END;
END;

{*************}

PROCEDURE RECHERCHE_CARACTERE(CHAINE:STRING;CARACTERE:CHAR;VAR POSITION_DU_CARACTERE:LONGINT;VAR NOMBRE:LONGINT);
{Gives the position of the first character CARACTERE and the number of CARACTERE in the string CHAINE}
{on regarde s'il existe dans une chaine CARACTERE, on donne le
NOMBRE de caractres "CARACTERES" et la position du premier (POSITION_CARACTERE)
POSITION_DU_CARACTERE=0 si rien trouv}
VAR COMPTEUR,LONGUEUR:LONGINT;
VAR CARAC:CHAR;
BEGIN
 {initialisation}
 POSITION_DU_CARACTERE:=0;
 NOMBRE:=0;
 LONGUEUR:=LENGTH(CHAINE);

 {dtection de la position du premier caractre et vrification du nombre de
 caractres (>0)}
 FOR COMPTEUR:=1 TO LONGUEUR DO
 BEGIN
  CARAC:=CHAINE[COMPTEUR];
  IF CARAC=CARACTERE THEN
  BEGIN
   POSITION_DU_CARACTERE:=COMPTEUR;
   BREAK; {on arrte la boucle}
  END;
 END;
 {si pas de CARACTERE, on a termin}
 IF POSITION_DU_CARACTERE=0 THEN EXIT;

 {dtermination du nombre de CARACTERES}
 FOR COMPTEUR:=1 TO LONGUEUR DO
 BEGIN
  CARAC:=CHAINE[COMPTEUR];
  IF CARAC=CARACTERE THEN INC(NOMBRE);
 END;
END;

{*************}

PROCEDURE EXTRACTION_PARAMETRES_SEPARES_PAR_CARACTERE(CHAINE:STRING;CARACTERE_DE_SEPARATION:CHAR;REDUCTION_PARAMETRE:BOOLEAN;
 VAR NOMBRE_DE_PARAMETRES:LONGINT;VAR PARAMETRES:TYPE_PARAMETRES);
{Extracts parameters separated by a character CARACTERE_DE_SEPARATION in the string CHAINE}
{ partir de CHAINE, on extrait les paramtres (sans le caractre de sparation) spars par des CARACTERE_DE_SEPARATION.
Si le dernier caractre n'est pas un caractre de sparation, on rajoute un caractre de sparation  la fin du message
Exemple avec / en sparation: 'az /er/ty'...on a 3 paramtres et non 2. De mme 'az' constitue un paramtre, mais '' a 0 paramtre.
On peut rduire chaque paramtre (on enlve les espaces et CHR(10) et CHR(13) initiaux et finaux)
On fournit le nombre de paramtres (max 255) dans NOMBRE_DE_PARAMETRES et les paramtres sont dans PARAMETRES.}
VAR COMPTEUR,LONGUEUR,POSITION_DU_CARACTERE:LONGINT;
BEGIN
 LONGUEUR:=LENGTH(CHAINE);
 IF LONGUEUR=0 THEN BEGIN NOMBRE_DE_PARAMETRES:=0; EXIT; END;

 {Si le dernier caractre n'est pas un caractre de sparation, on rajoute un caractre de sparation  la fin du message}
 IF CHAINE[LONGUEUR]<>CARACTERE_DE_SEPARATION THEN CHAINE:=CHAINE+CARACTERE_DE_SEPARATION;

 {nombre de paramtres et la position du premier CARACTERE_DE_SEPARATION}
 RECHERCHE_CARACTERE(CHAINE,CARACTERE_DE_SEPARATION,POSITION_DU_CARACTERE,NOMBRE_DE_PARAMETRES);

 {on extrait les paramtres}
 {on limite  255 si erreur...V. 4.26.1}
 IF NOMBRE_DE_PARAMETRES>255 THEN NOMBRE_DE_PARAMETRES:=255;
 FOR COMPTEUR:=1 TO NOMBRE_DE_PARAMETRES DO
 BEGIN
  {prise du paramtre}
  PARAMETRES[COMPTEUR]:=COPY(CHAINE,1,POSITION_DU_CARACTERE-1);
  {rduction si demand}
  IF REDUCTION_PARAMETRE=TRUE THEN REDUCTION_CHAINE(PARAMETRES[COMPTEUR]);
  {mise  jour chaine}
  DELETE(CHAINE,1,POSITION_DU_CARACTERE);
  {position du premier CARACTERE_DE_SEPARATION}
  RECHERCHE_POSITION_PREMIER_CARACTERE(CHAINE,CARACTERE_DE_SEPARATION,POSITION_DU_CARACTERE);
 END;
END;

{*************}

FUNCTION LE_CARACTERE_EST_UN_CHIFFRE(CARACTERE:CHAR):BOOLEAN;
{TRUE if the character is a figure and reversely}
{TRUE si le caractre est un chiffre et inversement}
VAR RESULTAT:BOOLEAN;
BEGIN
 IF (CARACTERE>='0') AND (CARACTERE<='9') THEN RESULTAT:=TRUE ELSE
  RESULTAT:=FALSE;
 LE_CARACTERE_EST_UN_CHIFFRE:=RESULTAT;
END;

{*************}

FUNCTION NOMBRE_ENTIER_POSITIF(CHAINE:STRING):BOOLEAN;
{TRUE if the string is a positive integer number and reversely}
{TRUE si la chaine est un nombre entier positif ou nul et inversement}
VAR COMPTEUR:LONGINT;
VAR SUCCES:BOOLEAN;
BEGIN
 SUCCES:=TRUE;{initialisation}
 FOR COMPTEUR:=1 TO LENGTH(CHAINE) DO
 BEGIN
  IF LE_CARACTERE_EST_UN_CHIFFRE(CHAINE[COMPTEUR])=FALSE THEN
  BEGIN
   SUCCES:=FALSE;
   BREAK;
  END;
 END;
 NOMBRE_ENTIER_POSITIF:=SUCCES;
END;

{*************}

PROCEDURE RECHERCHE_POSITION_SOUS_CHAINE_DANS_CHAINE(CHAINE:STRING;SOUS_CHAINE:STRING;VAR POSITION_SOUS_CHAINE:LONGINT;VAR SUCCES:BOOLEAN);
{Gives the position of the sub-string (SOUS-CHAINE) in the string (CHAINE)}
{on regarde s'il existe une sous-chaine dans une chaine, on donne la
position du premier caractre de la sous-chaine dans chaine (POSITION_SOUS_CHAINE)
Exemple: 'ABCDE' 'CD' est  la position 3
SUCCES=FALSE si rien trouv}
VAR COMPTEUR,LONGUEUR,LONGUEUR2:LONGINT;
VAR CHAINE_EXTRAITE:STRING;
BEGIN
 {initialisation}
 POSITION_SOUS_CHAINE:=0;
 LONGUEUR:=LENGTH(CHAINE);
 LONGUEUR2:=LENGTH(SOUS_CHAINE);

 {dtection de la position de la sous-chaine}
 FOR COMPTEUR:=1 TO LONGUEUR-LONGUEUR2+1 DO
 BEGIN
  CHAINE_EXTRAITE:=COPY(CHAINE,COMPTEUR,LONGUEUR2);
  IF CHAINE_EXTRAITE=SOUS_CHAINE THEN
  BEGIN
   POSITION_SOUS_CHAINE:=COMPTEUR;
   BREAK; {on arrte la boucle}
  END;
 END;
 {si pas de CARACTERE, on a termin}
 IF POSITION_SOUS_CHAINE=0 THEN SUCCES:=FALSE ELSE SUCCES:=TRUE;
END;


{**********}

PROCEDURE TRANSFORMATION_CHAINE_DWORD_EN_DWORD_AVEC_CONTROLE(CHAINE:STRING;VAR NOMBRE:DWORD;VAR REUSSITE:BOOLEAN);
{transforms a DWORD string in DWORD number}
{transforme le nombre DWORD en chane avec contrle. Si ce n'est un DWORD, REUSSITE=FALSE}
VAR V:INT64;
var Code: Integer;
BEGIN
 REUSSITE:=FALSE;{initialisation}
 REDUCTION_CHAINE(CHAINE);
 VAL(CHAINE,V,CODE);
 IF CODE<>0 THEN REUSSITE:=FALSE ELSE
 BEGIN
  IF (V>=0) AND (V<=4294967295) THEN
  BEGIN
   REUSSITE:=TRUE;
   NOMBRE:=V;
  END;
 END;
END;

{*********}

FUNCTION TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(THE_LONGWORD:STRING):DWORD;
{Inverse of the previous function, little Endian (weakest byte first)...LONGWORD is on 4 bytes
Example: CHR(128) CHR(187) CHR(0) CHR(0) gives 48000 and CHR(0) CHR(119) CHR(1) CHR(0) gives 96000}
VAR BYTE1,BYTE2,BYTE3,BYTE4:BYTE;
BEGIN
 BYTE1:=ORD(THE_LONGWORD[1]); {weakest byte}
 BYTE2:=ORD(THE_LONGWORD[2]);
 BYTE3:=ORD(THE_LONGWORD[3]);
 BYTE4:=ORD(THE_LONGWORD[4]); {strongest byte}
 RESULT:=BYTE4; RESULT:=RESULT*256*65536+BYTE3*65536+BYTE2*256+BYTE1;
 TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD:=RESULT;
END;

{*********}

FUNCTION TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(THE_WORD:WORD):STRING;
{little Endian (weakest byte first)...WORD is on 2 bytes, Example: 258 gives CHR(2) CHR(1) }
VAR BYTE1,BYTE2:BYTE;
VAR CHAR1,CHAR2:CHAR;
BEGIN
 BYTE1:=THE_WORD DIV 256; CHAR1:=CHR(BYTE1); {strongest byte}
 BYTE2:=THE_WORD MOD 256; CHAR2:=CHR(BYTE2); {weakest byte}
 TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN:=CHAR2+CHAR1;
END;

{***********}

FUNCTION TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(THE_LONGWORD:DWORD):STRING;
{little Endian (weakest byte first)...LONGWORD is on 4 bytes,
Example: 48000 gives CHR(128) CHR(187) CHR(0) CHR(0) and 96000 gives CHR(0) CHR(119) CHR(1) CHR(0)}
VAR WORD1,WORD2:WORD;
VAR STRING1,STRING2:STRING;
BEGIN
 WORD1:=THE_LONGWORD DIV 65536; STRING1:=TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(WORD1); {strongest word}
 WORD2:=THE_LONGWORD MOD 65536; STRING2:=TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(WORD2); {weakest word}
 TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN:=STRING2+STRING1;
END;

{*******}

procedure Non_blocking_Delay(msecs: DWORD);
var Stop:DWORD;
begin
  Stop := GetTickCount + msecs;
  while GetTickCount<Stop do
  begin
    Application.ProcessMessages;
    sleep(1);
  end
end;

{**********}

FUNCTION BUILD_A_SHORTSTRING(TEXT:STRING):STRING;
{for a TEXT with a length<=255}
VAR L:BYTE;
BEGIN
 L:=LENGTH(TEXT);
 BUILD_A_SHORTSTRING:=CHR(L)+TEXT;
END;

{***********}

PROCEDURE TRANSMISSION_TCP_IP_COMMAND(COMMAND:STRING;DISPLAYED:BOOLEAN);
{to the Multipsk server}
VAR STRING0:STRING;
BEGIN
 IF DISPLAYED=TRUE THEN
 BEGIN
  DateTime := Time; Date_Str := TimeToStr(DateTime);
  Form1.Memo_Log.Lines.Add(Date_Str+' TX: '+COMMAND);
  Form1.Memo_reception_Commands.Lines.Add(Date_str+' Command: '+COPY(COMMAND,1,100));
 END;
 {V.0.1, addition of the lenght of the command before the command (redundant information which can simplify reception)}
 STRING0:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(LENGTH(COMMAND));
 Form1.ClientSocket1.Socket.SendText(STRING0+COMMAND);
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the end of transmission of a command}
END;

{************}

procedure TForm1.ConnectionClick(Sender: TObject);
begin
 ClientSocket1.Host:='127.0.0.1';
 DateTime := Time; Date_str := TimeToStr(DateTime);
 Memo_Log.Lines.Add(Date_Str+': '+'Connexion asked');
 ClientSocket1.Open;
end;

{************}

procedure Disconnection_TCP_IP;
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 Form1.Memo_Log.Lines.Add(Date_str+' TX: '+'Disconnection asked');
 {Close the socket connexion}
 Form1.ClientSocket1.Close;
 Form1.Label_Client.Visible:=FALSE;
end;


{***********}

procedure TForm1.DisconnectionClick(Sender: TObject);
begin
 Disconnection_TCP_IP;
end;

{**********}

procedure TForm1.ClientSocket1Connect(Sender: TObject;Socket: TCustomWinSocket);
VAR COMMAND:STRING;
VAR INDICATIF,IP:STRING;VAR REUSSITE:BOOLEAN;
VAR LONGUEUR:BYTE;
VAR CAR:CHAR;
begin
 DateTime := Time; Date_str:= TimeToStr(DateTime);
 {This message specifies that the client is connected to the server}
 Memo_Log.Lines.Add(Date_str+': '+'Connected to '+Socket.RemoteHost);
 Label_Client.Visible:=TRUE;
 Form1.Connection.down:=TRUE;
 COMMAND:='Pversion'+CHR(1)+'0';
 TRANSMISSION_TCP_IP_COMMAND(COMMAND,TRUE);
 CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=TRUE;{now, after transmission of PVERSION, the control can begin}
 COMMAND_TO_BE_RECEIVED:='Pversion';{the first one to be received after connection and transmission of Pversion}
end;

{**********}

procedure TForm1.ClientSocket1Connecting(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 {This message specifies that the server is being searched by the client}
 Memo_Log.Lines.Add(Date_Str+': '+'The server is being searched');
end;

{**********}

procedure TForm1.ClientSocket1Disconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 {This message specifies that the client is going to disconnect from the server}
 Memo_Log.Lines.Add(Date_str+': '+'Disconnected');
 {initialisation}
 CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;
 Label_Client.Visible:=FALSE;

 {si le client TCP/IP est vu dconnect sans que le client Multipsk ait dconnect,
 c'est que le matre a coup la connexion sans avertissement, il faut donc revenir au fonctionnement normal, en RX partant de RX ou de TX}
 IF Disconnection.down=FALSE THEN
 BEGIN
  Disconnection.Down:=TRUE;
  ClientSocket1.Close;
  DISPLAY_INFORMATION_MESSAGE_ON_DISCONNECTION:=TRUE;  {display not done at this level as it blocks the disconnection}
 END ELSE
 BEGIN
  Form1.Disconnection.down:=TRUE;
 END;
end;


{**********}

PROCEDURE ANALYSE_Pversion_COMMAND_WITH_DATA_RECEIVED(VAR ERRONEOUS_COMMAND:BOOLEAN);
{The parameters must be entirely received before doing the analysis}
BEGIN
 {initialization}
 ERRONEOUS_COMMAND:=FALSE;

 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 {Multipsk 4.31.4 manages only the versions 0 and 1}{V.1}
 MP_VERSION:=ORD(CURRENT_STRING_RECEIVED[1]);
 IF (MP_VERSION<>0) AND (MP_VERSION<>1) THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{V.1}
 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,1);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 {The next requets is HWMPcapa}
 TRANSMISSION_TCP_IP_COMMAND('HWMPcapa',TRUE);
 {New command to be received}
 COMMAND_TO_BE_RECEIVED:='HWMPcapa';{the second one to be received after connection}
END;

{**********}

PROCEDURE ANALYSE_HWMPcapa_COMMAND_WITH_DATA_RECEIVED(VAR ERRONEOUS_COMMAND:BOOLEAN;VAR SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK:BOOLEAN;VAR THE_HARDWARE_TYPE:BYTE;VAR NOT_ENOUGH_CHARACTERS:BOOLEAN);
VAR COUNTER,LP1{for Length of parameter 1 (i.e. name)},LP2,LP3,LP4,LP5,LP6:LONGINT;
VAR NUMBER_OF_CHARACTERS_SUFFICIENT:BOOLEAN;
VAR name,model,the_message,Type_managed_by_the_MP,Sampling_rates_managed_by_the_MP,LO_Frequencies_managed_by_the_MP:STRING;
VAR GUI_received,GUI_to_send:CHAR;
VAR GUI_managed_by_the_MP:BOOLEAN;
VAR STRING_SERVER,STRING1,STRING2:STRING;
VAR NUMBER_OF_PARAMETERS_TYPE,NUMBER_OF_PARAMETERS_LO_FREQUENCY,NUMBER_OF_PARAMETERS_SR:LONGINT;
VAR PARAMETERS_TYPE,PARAMETERS_LO_FREQUENCY,PARAMETERS_SR:TYPE_PARAMETRES;
VAR OK_TYPE,OK_GUI,OK_SR,OK_INTERVAL:BOOLEAN;
VAR STRING_SAVED:STRING;
VAR LENGTH_RECEIVED:DWORD;

TYPE INTERVAL_OF_FREQUENCY=ARRAY[1..2] OF DWORD;
TYPE CONTAINER_TO_60_INTERVALS=RECORD
    NUMBER_OF_INTERVALS:BYTE;    { Dcalage du premier chiffre  mettre }
    ARRAY_OF_60_INTERVALS:ARRAY[1..60] OF INTERVAL_OF_FREQUENCY;{60 intervals is the maximum}
          END;
VAR LO_FREQUENCY_INTERVALS:CONTAINER_TO_60_INTERVALS;
VAR SAMPLING_RATE_INTERVALS:CONTAINER_TO_60_INTERVALS;
VAR INTERVAL:INTERVAL_OF_FREQUENCY;

{The parameters must be entirely received before doing the analysis}

 {!!!!!!!!!}

 PROCEDURE TRANSFORMATION_IN_INTERVAL(STRING1:STRING;VAR SUCCESS:BOOLEAN;VAR INTERVAL:INTERVAL_OF_FREQUENCY);
 {It can be received "48000", "96000" or "192000..292000". It will be given the interval in numbers.
  A sole frequency will have the interval frequency/frequency}
 VAR POSITION_TWO_DOTS:LONGINT;
 VAR OK1,OK2:BOOLEAN;
 VAR FR1,FR2:STRING;
 VAR FREQUENCY1,FREQUENCY2:DWORD;
 BEGIN
  SUCCESS:=FALSE;{initialization}

  RECHERCHE_POSITION_SOUS_CHAINE_DANS_CHAINE(STRING1,'..',POSITION_TWO_DOTS,OK1);
  IF OK1=TRUE THEN {type: "192000..292000"}
  BEGIN
   FR1:=COPY(STRING1,1,POSITION_TWO_DOTS-1);
   FR2:=COPY(STRING1,POSITION_TWO_DOTS+2,999);
   IF (NOMBRE_ENTIER_POSITIF(FR1)=TRUE) AND (NOMBRE_ENTIER_POSITIF(FR2)=TRUE) THEN {2 positive numbers}
   BEGIN
    TRANSFORMATION_CHAINE_DWORD_EN_DWORD_AVEC_CONTROLE(FR1,FREQUENCY1,OK1);{transformation in number with control}
    TRANSFORMATION_CHAINE_DWORD_EN_DWORD_AVEC_CONTROLE(FR2,FREQUENCY2,OK2);{transformation in number with control}
    IF (OK1=TRUE) AND (OK2=TRUE) THEN
    BEGIN
     {The minimum sampling rate is 2000 Hz. The maximum is 999 999 999 Hz+ control that FR1 and FR2 are in growing order}
     IF (FREQUENCY1>=2000) AND (FREQUENCY1<=999999999) AND (FREQUENCY2>=2000)  AND (FREQUENCY2<=999999999) AND (FREQUENCY1<=FREQUENCY2) THEN {success}
     BEGIN
      SUCCESS:=TRUE;
      INTERVAL[1]:=FREQUENCY1;
      INTERVAL[2]:=FREQUENCY2;
     END;
    END;
   END;
  END ELSE  {type: "48000"}
  BEGIN
   IF NOMBRE_ENTIER_POSITIF(STRING1)=TRUE THEN {positive number}
   BEGIN
    TRANSFORMATION_CHAINE_DWORD_EN_DWORD_AVEC_CONTROLE(STRING1,FREQUENCY1,OK1);{transformation in number with control}
    IF OK1=TRUE THEN
    BEGIN
     {The minimum sampling rate is 2000 Hz. The maximum is 999 999 999 Hz}
     IF (FREQUENCY1>=2000) AND (FREQUENCY1<=999999999) THEN {success}
     BEGIN
      SUCCESS:=TRUE;
      INTERVAL[1]:=FREQUENCY1;
      INTERVAL[2]:=FREQUENCY1;
     END;
    END;
   END;
  END;
 END;

 {!!!!!!!!!}

 FUNCTION IS_THE_INTERVAL_CONTAINS_THE_FREQUENCY(INTERVAL:INTERVAL_OF_FREQUENCY;FREQUENCY:DWORD):BOOLEAN;
 BEGIN
  IF (FREQUENCY>=INTERVAL[1]) AND (FREQUENCY<=INTERVAL[2]) THEN IS_THE_INTERVAL_CONTAINS_THE_FREQUENCY:=TRUE
  ELSE IS_THE_INTERVAL_CONTAINS_THE_FREQUENCY:=FALSE;
 END;

BEGIN {beginning of ANALYSE_HWMPcapa_COMMAND_WITH_DATA_RECEIVED}
 STRING_SAVED:=CURRENT_STRING_RECEIVED;{to keep CURRENT_STRING_RECEIVED in case there are no sufficiently characters}
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);
 LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);

 {initialization}
 ERRONEOUS_COMMAND:=FALSE; {Note: an erroneous error can only come from the programmation, as TCP/IP don't made any errors}
 NUMBER_OF_CHARACTERS_SUFFICIENT:=FALSE;
 SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK:=FALSE; OK_TYPE:=FALSE; OK_SR:=FALSE; OK_GUI:=FALSE;

 {Checking that the number of characters received is sufficient because the parameters must be entirely received before doing the analysis}
 {name}
 LP1:=ORD(CURRENT_STRING_RECEIVED[1]);
 IF LP1=0 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{at least one character}
 IF LENGTH_RECEIVED>(1+LP1) THEN {Yes enough+the next character}
 BEGIN
  {model}
  LP2:=ORD(CURRENT_STRING_RECEIVED[(1+LP1)+1]);
  IF LP2=0 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{at least one character}
  IF LENGTH_RECEIVED>(1+LP1+1+LP2) THEN {Yes enough+the next character}
  BEGIN
   {message}
   LP3:=ORD(CURRENT_STRING_RECEIVED[(1+LP1+1+LP2)+1]);{message can be void (LP3=0)}
   IF LENGTH_RECEIVED>(1+LP1+1+LP2+1+LP3) THEN {Yes enough+the next character}
   BEGIN
    {Type_managed_by_the_MP}
    LP4:=ORD(CURRENT_STRING_RECEIVED[(1+LP1+1+LP2+1+LP3)+1]);
    IF LP4<>1 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{V.1 only one character}
    IF LENGTH_RECEIVED>(1+LP1+1+LP2+1+LP3+1+LP4) THEN {Yes enough+the next character}
    BEGIN
     {LO_Frequencies_managed_by_the_MP}
     LP5:=ORD(CURRENT_STRING_RECEIVED[(1+LP1+1+LP2+1+LP3+1+LP4)+1]);
     IF LP5<4 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{at least 4 characters}
     IF LENGTH_RECEIVED>(1+LP1+1+LP2+1+LP3+1+LP4+1+LP5) THEN {Yes enough+the next character}
     BEGIN
      {Sampling_rates_managed_by_the_MP}
      LP6:=ORD(CURRENT_STRING_RECEIVED[(1+LP1+1+LP2+1+LP3+1+LP4+1+LP5)+1]);
      IF LP6<1 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{at least 1 character}
      {Yes enough+the next character which is the GUI_managed_by_the_MP}
      IF LENGTH_RECEIVED>(1+LP1+1+LP2+1+LP3+1+LP4+1+LP5+1+LP6) THEN NUMBER_OF_CHARACTERS_SUFFICIENT:=TRUE;
     END;
    END;
   END;
  END;
 END;

 {Are there sufficiently characters, if yes: immediate analysis is done, if not: it will be expected the next data}
 IF NUMBER_OF_CHARACTERS_SUFFICIENT=TRUE THEN {Yes enough}
 BEGIN
  {loading}
  name:=COPY(CURRENT_STRING_RECEIVED,2,LP1);
  model:=COPY(CURRENT_STRING_RECEIVED,3+LP1,LP2);
  the_message:=COPY(CURRENT_STRING_RECEIVED,4+LP1+LP2,LP3);
  Type_managed_by_the_MP:=COPY(CURRENT_STRING_RECEIVED,5+LP1+LP2+LP3,LP4);
  Sampling_rates_managed_by_the_MP:=COPY(CURRENT_STRING_RECEIVED,6+LP1+LP2+LP3+LP4,LP5);
  LO_Frequencies_managed_by_the_MP:=COPY(CURRENT_STRING_RECEIVED,7+LP1+LP2+LP3+LP4+LP5,LP6);
  GUI_received:=CURRENT_STRING_RECEIVED[7+LP1+LP2+LP3+LP4+LP5+LP6];{CHR(0) or CHR(1)}
  GUI_received:=CHR(ORD(GUI_received)+48);{"0" or "1"}

  {display}
  STRING_SERVER:='Server';
  STRING_SERVER:=STRING_SERVER+': '+name+' '+model+' '+the_message
  +' ("TYPE":'+Type_managed_by_the_MP+' "SR":'+Sampling_rates_managed_by_the_MP+' "FR."'+LO_Frequencies_managed_by_the_MP
  +' "GUI":'+GUI_received+')';
  Form1.Label_server.Caption:=STRING_SERVER; {display possibly complete}
  DateTime := Time; Date_Str:= TimeToStr(DateTime);
  Form1.Memo_Log.Lines.Add(Date_Str+' RX: '+STRING_SERVER);{display complete}

  {extraction analysis}

  {V.1 Type}
  {Multipsk must find the type 3, 4 or 5}
  IF Type_managed_by_the_MP='3' THEN BEGIN OK_TYPE:=TRUE; THE_HARDWARE_TYPE:=3; END ELSE
  IF Type_managed_by_the_MP='4' THEN BEGIN OK_TYPE:=TRUE; THE_HARDWARE_TYPE:=4; END ELSE
  IF Type_managed_by_the_MP='5' THEN BEGIN OK_TYPE:=TRUE; THE_HARDWARE_TYPE:=5; END;

  {GUI_managed_by_the_MP}
  IF (GUI_received='0') OR (GUI_received='1') THEN
  BEGIN
   OK_GUI:=TRUE;
   IF GUI_received='0' THEN GUI_managed_by_the_MP:=FALSE ELSE GUI_managed_by_the_MP:=TRUE;
  END;

  {SR}
  EXTRACTION_PARAMETRES_SEPARES_PAR_CARACTERE(Sampling_rates_managed_by_the_MP,'/',TRUE,NUMBER_OF_PARAMETERS_SR,PARAMETERS_SR);
  {Storage of all interval of sampling rates}
  SAMPLING_RATE_INTERVALS.NUMBER_OF_INTERVALS:=NUMBER_OF_PARAMETERS_SR;
  FOR COUNTER:=1 TO NUMBER_OF_PARAMETERS_SR DO
  BEGIN
   STRING1:=PARAMETERS_SR[COUNTER];
   TRANSFORMATION_IN_INTERVAL(STRING1,OK_INTERVAL,INTERVAL);
   IF OK_INTERVAL=TRUE THEN SAMPLING_RATE_INTERVALS.ARRAY_OF_60_INTERVALS[COUNTER]:=INTERVAL
   ELSE BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;
  END;
  {It must be found the 48000 Hz in the proposed interval}
  FOR COUNTER:=1 TO NUMBER_OF_PARAMETERS_SR DO
  BEGIN
   INTERVAL:=SAMPLING_RATE_INTERVALS.ARRAY_OF_60_INTERVALS[COUNTER];
   IF IS_THE_INTERVAL_CONTAINS_THE_FREQUENCY(INTERVAL,48000)=TRUE THEN BEGIN OK_SR:=TRUE; BREAK; END;
  END;

  IF (OK_TYPE=TRUE) AND (OK_GUI=TRUE) AND (OK_SR=TRUE) THEN {if not true, Multipsk will abandon the link}
  BEGIN
   SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK:=TRUE;
   {LO frequencies}
   EXTRACTION_PARAMETRES_SEPARES_PAR_CARACTERE(LO_Frequencies_managed_by_the_MP,'/',TRUE,NUMBER_OF_PARAMETERS_LO_FREQUENCY,PARAMETERS_LO_FREQUENCY);
   {Storage of all interval of LO frequencies}
   SAMPLING_RATE_INTERVALS.NUMBER_OF_INTERVALS:=NUMBER_OF_PARAMETERS_LO_FREQUENCY;
   FOR COUNTER:=1 TO NUMBER_OF_PARAMETERS_LO_FREQUENCY DO
   BEGIN
    STRING1:=PARAMETERS_LO_FREQUENCY[COUNTER];
    TRANSFORMATION_IN_INTERVAL(STRING1,OK_INTERVAL,INTERVAL);
    IF OK_INTERVAL=TRUE THEN LO_FREQUENCY_INTERVALS.ARRAY_OF_60_INTERVALS[COUNTER]:=INTERVAL
    ELSE BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;
   END;
  END;

  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,7+LP1+LP2+LP3+LP4+LP5+LP6);
  {Reinitialization}
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';

  {The answer is InitHWMP in type 3 or 4, 48000 Hz,the minimum frequency of the HW (it's just the initial choice, which can be modified later) and GUI_displayed}
  STRING1:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(48000);
  STRING2:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(LO_FREQUENCY_INTERVALS.ARRAY_OF_60_INTERVALS[1][1]);
  IF GUI_managed_by_the_MP=FALSE THEN GUI_to_send:=CHR(0) ELSE GUI_to_send:=CHR(1);{If managed by the MP, we choose to display the GUI}
  TRANSMISSION_TCP_IP_COMMAND('InitHWMP'+CHR(48+THE_HARDWARE_TYPE)+STRING1+STRING2+GUI_to_send,TRUE);

  {New command to be received}
  COMMAND_TO_BE_RECEIVED:='';{no precise command to receive. It could be TXHWdata or HW_Pause or Bug_on_P}
  Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for 'HWMPcapa'}
  NOT_ENOUGH_CHARACTERS:=FALSE;
 END ELSE
 BEGIN
  NOT_ENOUGH_CHARACTERS:=TRUE;
  CURRENT_STRING_RECEIVED:=STRING_SAVED;{to restore CURRENT_STRING_RECEIVED because there are no sufficiently characters}
 END;
END;

{**********}

PROCEDURE ANALYSE_Bug_on_P_COMMAND_WITH_DATA_RECEIVED(VAR NOT_ENOUGH_CHARACTERS:BOOLEAN);
VAR LENGTH_OF_THE_PARAMETER:LONGINT;
VAR TEXT_RECEIVED:STRING;
VAR STRING_SAVED:STRING;
VAR LENGTH_RECEIVED:DWORD;
{The parameters must be entirely received before doing the analysis}
BEGIN
 STRING_SAVED:=CURRENT_STRING_RECEIVED;{to keep CURRENT_STRING_RECEIVED in case there are no sufficiently characters}
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);
 LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);

 LENGTH_OF_THE_PARAMETER:=ORD(CURRENT_STRING_RECEIVED[1]);

 {Are there sufficiently characters, if yes: immediate analysis is done, if not: it will be expected the next data}
 IF LENGTH_RECEIVED>=(1+LENGTH_OF_THE_PARAMETER) THEN {Yes enough}
 BEGIN
  TEXT_RECEIVED:=COPY(CURRENT_STRING_RECEIVED,2,LENGTH_OF_THE_PARAMETER);
  Form1.Label_Info.Caption:=TEXT_RECEIVED;{Bug information displayed}

  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,1+LENGTH_OF_THE_PARAMETER);
  {Reinitialization}
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
  Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
  NOT_ENOUGH_CHARACTERS:=FALSE;
 END ELSE
 BEGIN
  NOT_ENOUGH_CHARACTERS:=TRUE;
  CURRENT_STRING_RECEIVED:=STRING_SAVED;{to restore CURRENT_STRING_RECEIVED because there are no sufficiently characters}
 END;
END;

{**********}

PROCEDURE ANALYSE_HW_Pause_COMMAND_WITH_DATA_RECEIVED;
VAR PAUSE_IN_SECONDS:LONGINT;
VAR TEXT_TO_DISPLAY:STRING;
{The parameters must be entirely received before doing the analysis}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 PAUSE_IN_SECONDS:=ORD(CURRENT_STRING_RECEIVED[1]);{1 to 5 normally}

 TEXT_TO_DISPLAY:='Pause requested';
 IF (PAUSE_IN_SECONDS<1) OR (PAUSE_IN_SECONDS>5) THEN {Error} BEGIN END
 ELSE TEXT_TO_DISPLAY:=TEXT_TO_DISPLAY+': '+CHR(PAUSE_IN_SECONDS+48)+' s';
 Form1.Label_Info.Caption:=TEXT_TO_DISPLAY;{information displayed}

 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,1);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_ADJUrate_COMMAND_WITH_DATA_RECEIVED;{V.1}
VAR TEXT_TO_DISPLAY:STRING;
{The parameters must be entirely received before doing the analysis}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 TCP_IP_FLOW_DATA_RATE:=ORD(CURRENT_STRING_RECEIVED[1]);{40 to 160 normally}
 TCP_IP_FLOW_DATA_RATE:=48000+(TCP_IP_FLOW_DATA_RATE-100)*200;{36000 to 60000 samples/s for 48000 nominal}

 STR(TCP_IP_FLOW_DATA_RATE,TEXT_TO_DISPLAY);
 Form1.Label_Info.Caption:='Data rate requested: '+TEXT_TO_DISPLAY+' samples/s';{information displayed}
 {...................................................................}
 {It is supposed that this new data rate has been taken into account} 
 {...................................................................}
 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,1);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_Set_HWLO_COMMAND_WITH_DATA_RECEIVED; {V.0.1}
VAR FREQUENCY_HZ:DWORD;
VAR TEXT_TO_DISPLAY,F:STRING;
{The parameter must be entirely received before doing the analysis}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 FREQUENCY_HZ:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(COPY(CURRENT_STRING_RECEIVED,1,4));

 TEXT_TO_DISPLAY:='Frequency';
 STR(FREQUENCY_HZ,F);
 TEXT_TO_DISPLAY:=TEXT_TO_DISPLAY+': '+F+' Hz';
 Form1.Label_Info.Caption:=TEXT_TO_DISPLAY;{information displayed}

 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,4);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_TXHWdata_COMMAND_WITH_DATA_RECEIVED(LENGTH_OF_THE_PARAMETER:DWORD);
{Reception of data}
VAR DATA_RECEIVED,IQ:STRING;
VAR COUNTER:DWORD;
VAR I,Q:SMALLINT;

 {!!!!!!!!}

 PROCEDURE TRANSFORMATION_OF_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_I_AND_Q(STRING1:STRING;VAR I:SMALLINT;VAR Q:SMALLINT);
 {Extraction of I and Q from a string of four characters
 It is substracted 32767 as expected by the protocol so to have I and Q between -32767 and 32767}
 //VAR BYTE1,BYTE2,BYTE3,BYTE4:LONGINT;
 VAR I_2c,Q_2c:WORD;{received in 2'complement}
 BEGIN
//  BYTE1:=ORD(THE_LONGWORD[1]); {weakest byte of I}
//  BYTE2:=ORD(THE_LONGWORD[2]); {strongest byte of I}
//  BYTE3:=ORD(THE_LONGWORD[3]); {weakest byte of Q}
//  BYTE4:=ORD(THE_LONGWORD[4]); {strongest byte of Q}
  I_2c:=ORD(STRING1[2])*256+ORD(STRING1[1]);
  Q_2c:=ORD(STRING1[4])*256+ORD(STRING1[3]);
  I:=I_2c; Q:=Q_2c;{2's complement number to signed number (from -32767 to 32767}
 END;

BEGIN
 {deletion of the command (8 characters)+the lenght on 4 bytes}
 DELETE(CURRENT_STRING_RECEIVED,1,12);

 {extraction of data}
 DATA_RECEIVED:=COPY(CURRENT_STRING_RECEIVED,1,LENGTH_OF_THE_PARAMETER);
 IF LENGTH_OF_THE_PARAMETER>0 THEN {LENGTH_OF_THE_PARAMETER=0 in case of empty TXHWdata (for a Hardware type 4)}
 BEGIN
  {this loop is optional depending on the way the user wishes to manage received data.
  It is reminded that the data is received in format 16 bits little endian, 2'complement}
  FOR COUNTER:=0 TO (LENGTH_OF_THE_PARAMETER DIV 4)-1 DO {LENGTH_OF_THE_PARAMETER DIV 4: number of parameters (IQ with 16 bits for I and Q}
  BEGIN
   IQ:=COPY(DATA_RECEIVED,COUNTER*4+1,4);
   TRANSFORMATION_OF_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_I_AND_Q(IQ,I,Q);
   {...................................................................}
  END;
  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,LENGTH_OF_THE_PARAMETER);
 END;

 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
 {Display useless after having being correctly connected and controlled (as it takes CPU)}
 TRANSMISSION_TCP_IP_COMMAND('Received',FALSE);

 {For the hardware Type 4, the MP regularly (each second) sent an empty TXHWdata to be sure that the link is effective. So here the CP shows this activity}
 IF HARDWARE_TYPE=4 THEN
 BEGIN
  IF Form1.NOMBRE_MILLIERS_RX.Caption='.' THEN Form1.NOMBRE_MILLIERS_RX.Caption:='..' ELSE
  IF Form1.NOMBRE_MILLIERS_RX.Caption='..' THEN Form1.NOMBRE_MILLIERS_RX.Caption:='...' ELSE
  IF Form1.NOMBRE_MILLIERS_RX.Caption='...' THEN Form1.NOMBRE_MILLIERS_RX.Caption:='....' ELSE
  IF Form1.NOMBRE_MILLIERS_RX.Caption='....' THEN Form1.NOMBRE_MILLIERS_RX.Caption:='.....' ELSE Form1.NOMBRE_MILLIERS_RX.Caption:='.';
 END;
END;

{**********}

PROCEDURE ANALYSE_Received_COMMAND;
{V.1 Just to express presence of the CP (and so to reintizialize the time counter}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION:STRING);
VAR LENGTH_INFORMATION:LONGINT;
BEGIN
 LENGTH_INFORMATION:=LENGTH(BUG_INFORMATION);
 IF LENGTH_INFORMATION>255 THEN
 BEGIN
  BUG_INFORMATION:=COPY(BUG_INFORMATION,1,255);
  LENGTH_INFORMATION:=255;
 END;
 Non_blocking_Delay(10);{a small delay by precaution}
 TRANSMISSION_TCP_IP_COMMAND('Bug_on_P'+CHR(LENGTH_INFORMATION)+BUG_INFORMATION,TRUE);
 Non_blocking_Delay(100);{Leave the message be transmitted}
 Disconnection_TCP_IP;
 INITIALIZATION_TCP_IP_EN_RX;
END;

{**********}

procedure TForm1.ClientSocket1Read(Sender: TObject; Socket: TCustomWinSocket);
{management of a received string of characters}
VAR COMPTEUR:DWORD;
VAR COMMAND,RECEIVED,STRING1:STRING;
VAR INITIAL_LENGTH,LENGTH_RECEIVED:LONGINT;
VAR ERRONEOUS_COMMAND,COMMAND_COMPLETELY_ANALYSED:BOOLEAN;
VAR BUG_INFORMATION:STRING;
VAR SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK:BOOLEAN;
VAR LENGTH_OF_THE_PARAMETER:DWORD;
VAR INSUFFICIENT_LENGTH_RECEIVED:BOOLEAN;

 {!!!!!!!!}
 PROCEDURE UPDATING_FOLLOWING_COMMAND_RECEPTION(COUNTER:LONGINT);
 BEGIN
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=TRUE;
  COMMAND_RECEIVED:=COMMAND;
  {V.0.1 updating of the string received, by deleting the part before the extracted command}
  DELETE(CURRENT_STRING_RECEIVED,1,COUNTER-1);
  LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 END;

 {!!!!!!!!}
 PROCEDURE MANAGEMENT_PART_OF_THE_TCP_IP_RECEPTION;
 VAR COUNTER:LONGINT;
 BEGIN
  COMMAND:='';{initialization}
  INSUFFICIENT_LENGTH_RECEIVED:=FALSE;

  {Analysis of the characters received to detect a command}
  IF TCP_IP_COMMAND_RECEPTION_IN_PROGRESS=FALSE THEN {New command}
  BEGIN
   FOR COUNTER:=1 TO LENGTH_RECEIVED-7 DO {-7 because the length of all commands is 8 (and the first character is number 1, not 0)}
   BEGIN
    COMMAND:=COPY(CURRENT_STRING_RECEIVED,COUNTER,8);
    IF ((COMMAND='Pversion') AND (COMMAND_TO_BE_RECEIVED='Pversion')) OR
       ((COMMAND='HWMPcapa') AND (COMMAND_TO_BE_RECEIVED='HWMPcapa')) OR
       (COMMAND='Bug_on_P') OR (COMMAND='HW_Pause') OR (COMMAND='Set_HWLO'){V.0.1} OR (COMMAND='TXHWdata') OR
       (COMMAND='Received') OR (COMMAND='ADJUrate') {V.1} THEN
    BEGIN
     UPDATING_FOLLOWING_COMMAND_RECEPTION(COUNTER);
     BREAK;
    END;
   END;
  END;
  IF TCP_IP_COMMAND_RECEPTION_IN_PROGRESS=FALSE THEN {failure to find a command}
  BEGIN
   {updating of the strings with the part analysed}
   DELETE(CURRENT_STRING_RECEIVED,1,LENGTH_RECEIVED-7);
   EXIT;
  END ELSE {a command has been received, which must be managed}
  BEGIN
   ERRONEOUS_COMMAND:=FALSE;{initialization}
   {The command and the parameters must be entirely received before doing the analysis}

   {In the following 8 is the length of the command}
   {the length for parameters for Pversion is one character}
   IF (COMMAND_RECEIVED='Pversion') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+1) THEN ANALYSE_Pversion_COMMAND_WITH_DATA_RECEIVED(ERRONEOUS_COMMAND)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the minimum length for parameters for HWMPcapa is 15 characters (2+2+1+2+5+2+1)}
   IF (COMMAND_RECEIVED='HWMPcapa') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+15) THEN
    BEGIN
     ANALYSE_HWMPcapa_COMMAND_WITH_DATA_RECEIVED(ERRONEOUS_COMMAND,SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK,HARDWARE_TYPE,INSUFFICIENT_LENGTH_RECEIVED);
     IF SERVER_PARAMETERS_COMPATIBLE_WITH_MULTIPSK=FALSE THEN {server not compatble with Multipsk}
     BEGIN
      {The client is warned, by a bug information, before being disconnected}
      BUG_INFORMATION:='Server not compatible with Multipsk which only manages type: 3 (I/Q 16 bits) or 4 and sampling rate: 48000 Hz)';
      INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION);
      EXIT;
     END;
    END ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the minimum length for parameter for Bug_on_P is one character
   Once the command complete with its parameters these 8 characters are deleted}
   IF (COMMAND_RECEIVED='Bug_on_P') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+1) THEN ANALYSE_Bug_on_P_COMMAND_WITH_DATA_RECEIVED(INSUFFICIENT_LENGTH_RECEIVED)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the length for parameters for HW_Pause is one character}
   IF (COMMAND_RECEIVED='HW_Pause') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+1) THEN ANALYSE_HW_Pause_COMMAND_WITH_DATA_RECEIVED
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the length for parameter for Set_HWLO is 4 characters (one LONGWORD)}
   IF (COMMAND_RECEIVED='Set_HWLO') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+4) THEN ANALYSE_Set_HWLO_COMMAND_WITH_DATA_RECEIVED {V.0.1}
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the length for parameter for TXHWdata must be determined first. To received the length parameter, 4 characters are necessary}
   IF (COMMAND_RECEIVED='TXHWdata') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+4) THEN
    BEGIN
     LENGTH_OF_THE_PARAMETER:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(COPY(CURRENT_STRING_RECEIVED,8+1,4));
     IF LENGTH_RECEIVED>=(LENGTH_OF_THE_PARAMETER+8+4) THEN ANALYSE_TXHWdata_COMMAND_WITH_DATA_RECEIVED(LENGTH_OF_THE_PARAMETER)
     ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
    END ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {V.1 the length for parameters for ADJUrate is one character (the rate)}
   IF (COMMAND_RECEIVED='ADJUrate') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+1) THEN ANALYSE_ADJUrate_COMMAND_WITH_DATA_RECEIVED
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   IF (COMMAND_RECEIVED='Received') THEN ANALYSE_Received_COMMAND;

   IF ERRONEOUS_COMMAND=TRUE THEN
   BEGIN
    {The client is warned, by a bug information, before being disconnected}
    BUG_INFORMATION:='Incorrect command: '+COMMAND+CURRENT_STRING_RECEIVED;
    INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION);
    EXIT;
   END;
  END;
 END;

begin
 {updating}
 RECEIVED:=Socket.ReceiveText;
 {V.0.1, The 4 four first characters are the lenght of the command: this length is received but is ignored here, but could be used by programs, as it can simply reception)}
 {V.1 Display except "Received" and TXHWdata'}
 IF (COPY(RECEIVED,5{V.0.1},8)<>'TXHWdata') AND (COPY(RECEIVED,5{V.0.1},8)<>'Received') THEN Form1.Memo_reception_Text.Lines.Add(Date_str+': '+COPY(RECEIVED,5,50));

 {**********here begin the short saving and display part*********}
 {Display of the Kdata received}
 INC(NUMBER_OF_CHARACTERS_RECEIVED,LENGTH(RECEIVED));
 COMPTEUR:=NUMBER_OF_CHARACTERS_RECEIVED DIV 1000;
 IF COMPTEUR>PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED THEN
 BEGIN
  STR(COMPTEUR,STRING1);
  NOMBRE_MILLIERS_RX.Caption:='N RX(Kdata)='+STRING1;
  PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED:=COMPTEUR;//rinitialisation
 END;

 {Updating of STRING_RECEIVED_AND_SAVED}
 STRING_RECEIVED_AND_SAVED:=STRING_RECEIVED_AND_SAVED+RECEIVED;

 IF MANAGEMENT_TCP_IP_RECEPTION_FINISHED=FALSE THEN EXIT; {it is necessary to have finished the process before to re-begin a new one}
 {**********here begin the management part*********}
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=FALSE;

 {We must work with a "private" variable to be independant of STRING_RECEIVED_AND_SAVED
  in case of multiple call of this procedure ClientSocket1Read}
 CURRENT_STRING_RECEIVED:=STRING_RECEIVED_AND_SAVED;
 LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 INITIAL_LENGTH:=LENGTH_RECEIVED;
 IF LENGTH_RECEIVED<8 THEN BEGIN MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE; EXIT; END;{minimum for a single command: 8 characters}
 REPEAT
  INSUFFICIENT_LENGTH_RECEIVED:=FALSE;
  MANAGEMENT_PART_OF_THE_TCP_IP_RECEPTION;
  {updating}
  LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 UNTIL (LENGTH_RECEIVED<8) OR (INSUFFICIENT_LENGTH_RECEIVED=TRUE); {minimum for a single command: 8 characters}
 {management of the STRING_RECEIVED_AND_SAVED string according to the number of bytes managed}
 IF INITIAL_LENGTH-LENGTH_RECEIVED>0 THEN DELETE(STRING_RECEIVED_AND_SAVED,1,INITIAL_LENGTH-LENGTH_RECEIVED);
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE;
end;

{**********}

procedure TForm1.ClientSocket1Error(Sender: TObject;Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
var ErrorMsg: string;
begin
  {definition of the error message according to the error code}
  case ErrorCode of
   10004 : ErrorMsg:='Interrupted function call.';
   10013 : ErrorMsg:='Permission denied.';
   10014 : ErrorMsg:='Bad address.';
   10022 : ErrorMsg:='Invalid argument.';
   10024 : ErrorMsg:='Too many open files. Actually, it''s too many open sockets.';
   10035 : ErrorMsg:='Resource temporarily unavailable.';
   10036 : ErrorMsg:='Operation now in progress. A blocking operation is in progress.';
   10037 : ErrorMsg:='Operation already in progress.';
   10038 : ErrorMsg:='Socket operation on non-socket.';
   10039 : ErrorMsg:='Destination address required.';
   10040 : ErrorMsg:='Message too long.';
   10041 : ErrorMsg:='Protocol wrong type for socket.';
   10042 : ErrorMsg:='Bad protocol option.';
   10043 : ErrorMsg:='Protocol not supported.';
   10044 : ErrorMsg:='Socket type not supported.';
   10045 : ErrorMsg:='Operation not supported.';
   10046 : ErrorMsg:='Protocol family not supported.';
   10047 : ErrorMsg:='Address family not supported by protocol family.';
   10048 : ErrorMsg:='Address is already in use. If you attempt to set up a server on a port that is already is in use, you will get this error.';
   10049 : ErrorMsg:='Cannot assign requested address.';
   10050 : ErrorMsg:='Network is down.';
   10051 : ErrorMsg:='Network is unreachable.';
   10052 : ErrorMsg:='Network dropped connection on reset.';
   10053 : ErrorMsg:='Software caused the connection to abort. A connection that has been made was aborted, usually due to connection or protocol error.';
   10054 : ErrorMsg:='Connection reset by peer. This occurs when an established connection is shut down for some reason by the remote computer.';
   10055 : ErrorMsg:='No buffer space available.';
   10056 : ErrorMsg:='Socket is already connected.';
   10057 : ErrorMsg:='Socket is not connected.';
   10058 : ErrorMsg:='Cannot send after socket shutdown.';
   10060 : ErrorMsg:='Connection timed out.';
   10061 : ErrorMsg:='Connection refused. You will usually see this error when a server refuses a connection from a client, because the server is not listening on that port.';
   10064 : ErrorMsg:='Host is down.';
   10065 : ErrorMsg:='No route to host.';
   10067 : ErrorMsg:='Too many processes.';
   10091 : ErrorMsg:='Network subsystem is unavailable.';
   10092 : ErrorMsg:='Unsupported version of WINSOCK.DLL.';
   10093 : ErrorMsg:='Successful WSAStartup not yet performed. TCP networking has not been initialized on your computer.';
   10094 : ErrorMsg:='Graceful shutdown in progress.';
   11001 : ErrorMsg:='Host not found. (DNS error.)';
   11002 : ErrorMsg:='Non-authoritative host not found. Temporary DNS error.';
   11003 : ErrorMsg:='Non-recoverable error. (DNS error.)';
   11004 : ErrorMsg:='Valid name, no data record of requested type. (DNS error).';
  else
   ErrorMsg := 'Unknown socket error.';
  end;

  ErrorMsg :=CHR(13)+CHR(10)+'Socket Error n' + IntToStr(ErrorCode) + ' : ' + ErrorMsg;
  {The error is managed}
  ErrorCode := 0;
  {definition of the error type}
  case ErrorEvent of
        eeSend          : ErrorMsg := 'Writing error. ' + ErrorMsg;
        eeReceive       : ErrorMsg := 'Reading error. ' + ErrorMsg;
        eeConnect       : ErrorMsg := 'Connecting error. ' + ErrorMsg;
        eeDisconnect : ErrorMsg := 'Disconnecting error. ' + ErrorMsg;
        eeAccept         : ErrorMsg := 'Accepting error. ' + ErrorMsg;
  else
        // erreur inconnue
        ErrorMsg := 'Unknown error. ' + ErrorMsg;
  end;

 DateTime := Time; Date_Str:= TimeToStr(DateTime);
 {This message specifies that the client is going to disconnect from the server}
 Memo_Log.Lines.Add(Date_Str+': '+ErrorMsg);
 Form1.Disconnection.down:=TRUE;
 {abandon}
 Label_Client.Visible:=FALSE;
end;


{************}

procedure TForm1.Timer1Timer(Sender: TObject);
VAR BUG_INFORMATION:STRING;
VAR LENGTH_INFORMATION:WORD;
VAR NUMBER_OF_IQ_SAMPLES_TO_SEND:DWORD;
VAR DELTA_TIME,NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT:DWORD;
VAR STRING_TO_SEND,STRING0:STRING;
VAR NUMBER_OF_PERIODS_OF_SIGNAL_TO_SEND,COUNTER:DWORD;
VAR Min,Max:SMALLINT;
VAR Final_min,Final_max:WORD;{signed number to 2's complement number}
VAR COMPTEUR:DWORD;

 {!!!!!!!!}

 PROCEDURE TRANSMISSION_OF_I_Q(I,Q:WORD);
 VAR STRING1:STRING;
 BEGIN
  STRING1:=CHR(I MOD 256)+CHR(I DIV 256)+CHR(Q MOD 256)+CHR(Q DIV 256);
  STRING_TO_SEND:=STRING_TO_SEND+STRING1;
 END;

begin
 {The CP client, must consider that after 10 seconds waiting for a MP event the MP has some problem and the CP will disconnect the MP.}
 IF (CONTROL_OF_MAXIMUM_DELAY_ACTIVATED=TRUE) AND ((GetTickCount-Initial_time_of_an_event_for_waiting_delay)>Maximum_delay_to_receive_the_expected_event) THEN
 BEGIN
  CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;{to avoid re-coming}
  DateTime := Time; Date_Str := TimeToStr(DateTime);
  Memo_Log.Lines.Add(Date_Str+': '+'Server absent from more than 10 seconds, so automatic disconnection');
  {The server is warned, by a bug information before being disconnected}
  BUG_INFORMATION:='Disconnection from the server due to a too long waiting time';
  INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION);
 END;

 IF DISPLAY_INFORMATION_MESSAGE_ON_DISCONNECTION=TRUE THEN
 BEGIN
  {done at this level. If not block the disconnection}
  DISPLAY_INFORMATION_MESSAGE_ON_DISCONNECTION:=FALSE;
  MessageDlg('Disconnection from the SDR TCP/IP server. Automatic return to normal working from the sound card',mtInformation,[mbOK],0);
 END;

 {Data transmission V.1}
 {It is important to not re-enter in the procudure before the end, so use of Timer1.Enabled}
 IF SIMULATED_DATA_TO_SEND=TRUE THEN {Simulation of data transmitted requested}
 {It is simulated a square signal at a frequency of 8000 Hz with a SR (sampling frequency) of 48000 Hz and 16 bits for I and Q.
 So the period of the signal will equal to 8 samples SR=48000/7000.
 With a period of 8 samples the Q is out of phase of 2 samples (Pi/2) from I.
 However this permits to have two traces on Multipsk, on each side of the SdR waterfall
 The timer must be the most quick possible to send packets of 3000 bytes or less (consider 4000 byts as a maximum).
 Above, the CPU load increases up to possibly freezes}
 BEGIN
  {It is important to not re-enter in the procudure before the end, so use of Timer1.Enabled}
  Timer1.Enabled:=FALSE;
  {First call to this function, initialization}
  IF INITIAL_TIME_OF_DATA_TRANSMISSION=0 THEN
  BEGIN
   INITIAL_TIME_OF_DATA_TRANSMISSION:=GetTickCount;
   NUMBER_OF_IQ_SAMPLES_SENT:=0;
  END ELSE
  {Next calls}
  BEGIN
   DELTA_TIME:=GetTickCount-INITIAL_TIME_OF_DATA_TRANSMISSION;
   NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT:=48*DELTA_TIME;{normally: (48000*DELTA_TIME) DIV 1000};
   NUMBER_OF_IQ_SAMPLES_TO_SEND:=NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT-NUMBER_OF_IQ_SAMPLES_SENT;
   IF NUMBER_OF_IQ_SAMPLES_TO_SEND=0 THEN EXIT;
   {Transmission with Q delayed from I of Pi/2}
   {NUMBER_OF_IQ_SAMPLES_TO_SEND DIV 7 is the number of periods (with 7 samples/period at 7000 Hz}
   STRING_TO_SEND:='';

   {The initial signed signal is symmetrical between -32767  32767. It is produced here a square signal
   so between min and max}
   Min:=-32767; Max:=32767;
   Final_min:=Min;    Final_max:=Max; {signed number to 2's complement number: SMALLINIT to WORD}
   FOR COUNTER:=1 TO (NUMBER_OF_IQ_SAMPLES_TO_SEND DIV 8) DO
   BEGIN
    {for test: TRANSMISSION_OF_I_Q(12336,12336); TRANSMISSION_OF_I_Q(12593,12593); TRANSMISSION_OF_I_Q(12850,12850); TRANSMISSION_OF_I_Q(13107,13107);
    TRANSMISSION_OF_I_Q(13364,13364); TRANSMISSION_OF_I_Q(13621,13621); TRANSMISSION_OF_I_Q(13878,13878); TRANSMISSION_OF_I_Q(14135,14135);}
    {Sqaure wave signal}
    TRANSMISSION_OF_I_Q(Final_min,Final_max);
    TRANSMISSION_OF_I_Q(Final_min,Final_max);
    TRANSMISSION_OF_I_Q(Final_min,Final_min);
    TRANSMISSION_OF_I_Q(Final_min,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_max);
    TRANSMISSION_OF_I_Q(Final_max,Final_max);
   END;
   {16 bits for I and Q, so 32 bits for I+Q or 4 characters}
   STRING0:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(NUMBER_OF_IQ_SAMPLES_TO_SEND*4);
   {Display useless after having being correctly connected and controlled (as it takes CPU)}
   IF Form1.Connection.down=TRUE THEN{in case of disconnection while in the timer}
   BEGIN
    TRANSMISSION_TCP_IP_COMMAND('TXHWdata'+STRING0+STRING_TO_SEND,FALSE);
    INC(NUMBER_OF_IQ_SAMPLES_SENT,NUMBER_OF_IQ_SAMPLES_TO_SEND);

    {for test: STR(NUMBER_OF_IQ_SAMPLES_TO_SEND,STRING0); Button1.Caption;}

    {Display of the Kdata transmitted}
    INC(NUMBER_OF_CHARACTERS_TRANSMITTED,NUMBER_OF_IQ_SAMPLES_TO_SEND*4);
    COMPTEUR:=NUMBER_OF_CHARACTERS_TRANSMITTED DIV 1000;
    IF COMPTEUR>PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED THEN
    BEGIN
     STR(COMPTEUR,STRING0);
     NOMBRE_MILLIERS_TX.Caption:='N TX(Kdata)='+STRING0;
     PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED:=COMPTEUR;//rinitialisation
    END;

    {To avoid to pass the DWORD length}
    IF NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT>480000000 THEN
    BEGIN
     INC(INITIAL_TIME_OF_DATA_TRANSMISSION,10000000);
     DEC(NUMBER_OF_IQ_SAMPLES_SENT,480000000);
    END;
   END;
  END;
  Timer1.Enabled:=TRUE;{Timer ready now that it is finished}
 END;
end;

{*********}

procedure TForm1.FormActivate(Sender: TObject);
begin
 Memo_reception_Text.setfocus;
 INITIALIZATION_TCP_IP_EN_RX;
end;

{***********}

procedure TForm1.Manual_commandClick(Sender: TObject);
VAR COMMAND:STRING;
begin
 COMMAND:=Edit1.Text;
 TRANSMISSION_TCP_IP_COMMAND(COMMAND,TRUE);
end;

{***********}

procedure TForm1.Send_frequencyClick(Sender: TObject);
VAR STRING1:STRING;
VAR FREQUENCY_IN_HZ:DWORD;
begin
 FREQUENCY_IN_HZ:=14071000;{for example, 14.071 MHz}
 STRING1:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(FREQUENCY_IN_HZ);
 TRANSMISSION_TCP_IP_COMMAND('Set_HWLO'+STRING1,TRUE);
end;

{***********}

procedure TForm1.Send_TX_and_dataClick(Sender: TObject);
begin
 IF (Send_TX_and_data.down=TRUE) AND (HARDWARE_TYPE=5) THEN {XCVR}
 BEGIN
  RECEPTION_DATA_IN_PROGRESS:=FALSE; {TX now}
  TRANSMISSION_TCP_IP_COMMAND('SwitchTX',TRUE);
  SIMULATED_DATA_TO_SEND:=TRUE;
  INITIAL_TIME_OF_DATA_TRANSMISSION:=0;
 END ELSE
 BEGIN
  RECEPTION_DATA_IN_PROGRESS:=TRUE; {RX}
  SIMULATED_DATA_TO_SEND:=FALSE;
  Send_TX_and_data.down:=FALSE;
 END;
end;

end.
