unit Form_Server;
{Historic:

Revision 1 of the 24/03/2018  ("V.1"), mainly management of the RX/TX interface:
* SwitchTX, SwitchRX and TXHWdata procedures
* new HARDWARE_TYPE = 5 (XCVR)
* reception of simulated data sent by the client
* small modifications to be homogeneous with the client program

Revision 0.1 of the 20/03/2018 ("V.1"), mainly:
* addition of the lenght of the command before the command (redundant information which can simplify reception)
(in TRANSMISSION_TCP_IP_COMMAND)
* modification of the default TCP/IP port (50001 instead 3020)
* example of frequency transmission
* management of the optional procedure Set_HWLO (TX)
* removal of the management of the procedures Set_HWSR Show_GUI Hide_GUI (no more in 0.1 version)
This server can be interfaced with the simulation of a client SDR TCP/IP INTERFACE VERSION 0.1

Revision 0 of the 24/02/2017:
 * original issue for a server SDR TCP/IP INTERFACE VERSION 0 (not applicable for higher versions)
   applicable to MULTIPSK V.4.31.4, which manages only the mandatory part of the version 0 of
   the protocol, i.e.:
    * the defaut sampling frequency (48000 Hz),
    * the default type managed: 3 (16 bits, IQ)
    * none MP GUI
    * the mandatory procedures for an MP: Pversion (RX/TX) HWMPcapa (RX/TX) TxHWdata (TX) Received (RX) Bug_on_P (RX/TX) Set_HWLO (RX)
    However, the optional rocedure: HW_pause is also managed.
   This server can be also interfaced with the simulation of a client SDR TCP/IP INTERFACE VERSION 0

   Here this pseudo-server Version 0 of the protocol manages the minimum for Multipsk V.4.31.4,
   so it is only able of a sampling frequency of 48000 Hz (default frequency) and only able
   to manage the default type managed: 3 (16 bits, IQ)

   The number of data transmitted is displayed in the blue label (in Kdata).

   This server manages also the Hardware type 4 (for frequency control only), also managed by MULTIPSK V.4.31.4.
}

{APPTYPE CONSOLE}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ScktComp, Buttons, ComCtrls,ShellApi,
  ExtCtrls;

type
  TForm1 = class(TForm)
    GroupBox3: TGroupBox;
    Memo_Reception_Text: TMemo;
    GroupBox2: TGroupBox;
    GroupBox4: TGroupBox;
    Memo_Log: TMemo;
    Connection: TSpeedButton;
    Disconnection: TSpeedButton;
    Edit1: TEdit;
    Manual_command: TButton;
    Label5: TLabel;
    GroupBox7: TGroupBox;
    Memo_reception_commands: TMemo;
    ServerSocket1: TServerSocket;
    Timer1: TTimer;
    Label_Info: TLabel;
    Pause: TSpeedButton;
    HW_problem: TSpeedButton;
    TX_Data: TSpeedButton;
    Label_simulation: TLabel;
    Label_bug: TLabel;
    NOMBRE_MILLIERS_TX: TLabel;
    HW_TYPE_3: TSpeedButton;
    Label1: TLabel;
    HW_TYPE_4: TSpeedButton;
    Frequency: TSpeedButton;
    HW_TYPE_5: TSpeedButton;
    NOMBRE_MILLIERS_RX: TLabel;
    Adjust_speed: TSpeedButton;
    procedure ConnectionClick(Sender: TObject);
    procedure DisconnectionClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure Manual_commandClick(Sender: TObject);
    procedure ServerSocket1Accept(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocket1ClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocket1ClientDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocket1ClientError(Sender: TObject;
      Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
      var ErrorCode: Integer);
    procedure ServerSocket1ClientRead(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocket1Listen(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure Timer1Timer(Sender: TObject);
    procedure HW_problemClick(Sender: TObject);
    procedure PauseClick(Sender: TObject);
    procedure TX_DataClick(Sender: TObject);
    procedure HW_TYPE_3Click(Sender: TObject);
    procedure HW_TYPE_4Click(Sender: TObject);
    procedure FrequencyClick(Sender: TObject);
    procedure HW_TYPE_5Click(Sender: TObject);
    procedure Adjust_rateClick(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;


procedure INITIALIZATION_AND_CONNECTION;
procedure INITIALIZATION_WITHOUT_CONNECTION;
procedure Connection_TCP_IP;

implementation

VAR Datetime:TDateTime;
var Date_Str: string;
VAR TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:BOOLEAN;
{The MP server must consider that after 15 seconds waiting for a CP event
(end of transmission of a command or beginning of transmission of a new command after having transmitted Pversion),
 the CP has some problem (often as a freezing) and the MP will disconnect the CP.
 Just before the disconnection, it will be transmitted the procedure Bug_on_P
 with the parameter "Disconnection due to a too long waiting time".}
VAR CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:BOOLEAN;
CONST Maximum_delay_to_receive_the_expected_event=15000;{ms, so 15 seconds}
VAR Time_waiting:DWORD;{to compare with Maximum_delay_to_receive_the_expected_event}
VAR Initial_time_of_an_event_for_waiting_delay:DWORD;
VAR COMMAND_RECEIVED:STRING;
VAR COMMAND_TO_BE_RECEIVED:STRING;
VAR CURRENT_STRING_RECEIVED:STRING;
VAR CP_VERSION:CHAR; {Multipsk 4.31.4 manages only the mandatory part of the version 0}
VAR SIMULATED_DATA_TO_SEND,DATA_MUST_BE_SENT:BOOLEAN;
VAR INITIAL_TIME_OF_DATA_TRANSMISSION:DWORD;
VAR NUMBER_OF_IQ_SAMPLES_SENT:DWORD;
VAR CLIENT_CONNECTED:BOOLEAN;
VAR PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED,NUMBER_OF_CHARACTERS_TRANSMITTED:DWORD;
VAR SERVER_IN_SERVICE:BOOLEAN;
VAR HARDWARE_TYPE:BYTE;{3, 4 or 5}
VAR Initial_time_of_TXHWdata_transmission:DWORD;
{V.1}
VAR PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED,NUMBER_OF_CHARACTERS_RECEIVED:DWORD;{V.1}
VAR RECEPTION_DATA_IN_PROGRESS:BOOLEAN;
VAR MANAGEMENT_TCP_IP_RECEPTION_FINISHED:BOOLEAN;{to avoid re-coming}
VAR STRING_RECEIVED_AND_SAVED:STRING;

{$R *.dfm}

{*********}

FUNCTION TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(THE_LONGWORD:STRING):DWORD;
{Inverse of the previous function, little Endian (weakest byte first)...LONGWORD is on 4 bytes
Example: CHR(128) CHR(187) CHR(0) CHR(0) gives 48000 and CHR(0) CHR(119) CHR(1) CHR(0) gives 96000}
VAR BYTE1,BYTE2,BYTE3,BYTE4:BYTE;
BEGIN
 BYTE1:=ORD(THE_LONGWORD[1]); {weakest byte}
 BYTE2:=ORD(THE_LONGWORD[2]);
 BYTE3:=ORD(THE_LONGWORD[3]);
 BYTE4:=ORD(THE_LONGWORD[4]); {strongest byte}
 RESULT:=BYTE4; RESULT:=RESULT*256*65536+BYTE3*65536+BYTE2*256+BYTE1;
 TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD:=RESULT;
END;

{*********}

FUNCTION TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(THE_WORD:WORD):STRING;
{little Endian (weakest byte first)...WORD is on 2 bytes, Example: 258 gives CHR(2) CHR(1) }
VAR BYTE1,BYTE2:BYTE;
VAR CHAR1,CHAR2:CHAR;
BEGIN
 BYTE1:=THE_WORD DIV 256; CHAR1:=CHR(BYTE1); {strongest byte}
 BYTE2:=THE_WORD MOD 256; CHAR2:=CHR(BYTE2); {weakest byte}
 TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN:=CHAR2+CHAR1;
END;

{***********}

FUNCTION TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(THE_LONGWORD:DWORD):STRING;
{little Endian (weakest byte first)...LONGWORD is on 4 bytes,
Example: 48000 gives CHR(128) CHR(187) CHR(0) CHR(0) and 96000 gives CHR(0) CHR(119) CHR(1) CHR(0)}
VAR WORD1,WORD2:WORD;
VAR STRING1,STRING2:STRING;
BEGIN
 WORD1:=THE_LONGWORD DIV 65536; STRING1:=TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(WORD1); {strongest word}
 WORD2:=THE_LONGWORD MOD 65536; STRING2:=TRANSFORMATION_WORD_IN_CHARACTERS_LITTLE_ENDIAN(WORD2); {weakest word}
 TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN:=STRING2+STRING1;
END;

{*******}

procedure Non_blocking_Delay(msecs: DWORD);
var Stop:DWORD;
begin
  Stop := GetTickCount + msecs;
  while GetTickCount<Stop do
  begin
    Application.ProcessMessages;
    sleep(1);
  end
end;

{**********}

FUNCTION BUILD_A_SHORTSTRING(TEXT:STRING):STRING;
{for a TEXT with a length<=255}
VAR L:BYTE;
BEGIN
 L:=LENGTH(TEXT);
 BUILD_A_SHORTSTRING:=CHR(L)+TEXT;
END;

{***********}

PROCEDURE TRANSMISSION_TCP_IP_COMMAND(COMMAND:STRING;DISPLAYED:BOOLEAN);
{to the Multipsk server}
VAR STRING0:STRING;
BEGIN
 {WRITELN(COPY(COMMAND,1,8));}
 IF (CLIENT_CONNECTED=FALSE) OR (SERVER_IN_SERVICE=FALSE) THEN EXIT;
 {display the command, limited to the 100 first characters, useless for TXHWdata}
 IF DISPLAYED=TRUE THEN
 BEGIN
  DateTime := Time; Date_str := TimeToStr(DateTime);
  Form1.Memo_reception_Commands.Lines.Add(Date_str+' Command: '+COPY(COMMAND,1,100));
 END;

 {V.0.1, addition of the lenght of the command before the command (redundant information which can simplify reception)}
 STRING0:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(LENGTH(COMMAND));
 COMMAND:=STRING0+COMMAND;
 Form1.ServerSocket1.Socket.Connections[0].SendText(COMMAND);
END;


{************}

procedure Connection_TCP_IP;
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 Form1.Memo_Log.Lines.Add(Date_Str+': '+'Link opening');
 Form1.ServerSocket1.Port:=50001; //Define the port used (50001 to 50020)
 Form1.ServerSocket1.Open; //Open the socket connection
 SERVER_IN_SERVICE:=TRUE; 
end;

{************}

procedure TForm1.ConnectionClick(Sender: TObject);
begin
 Connection_TCP_IP;
end;

{***********}

procedure Disconnection_TCP_IP;
begin
 SERVER_IN_SERVICE:=FALSE;
 DateTime := Time; Date_str := TimeToStr(DateTime);
 Form1.Memo_Log.Lines.Add(Date_Str+': '+'Disconnexion asked');
 {Close the socket connexion}
 Form1.ServerSocket1.Close;
 INITIALIZATION_WITHOUT_CONNECTION;
end;

{***********}

procedure TForm1.DisconnectionClick(Sender: TObject);
begin
 Disconnection_TCP_IP;
end;

{**********}

procedure TForm1.ServerSocket1Accept(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 {This message indicates the distant IP address of the client connecting to the server}
 Memo_Log.Lines.Add(Date_Str+': '+'Connected to '+Socket.RemoteAddress);
 CLIENT_CONNECTED:=TRUE;
 HW_TYPE_3.Enabled:=FALSE; HW_TYPE_4.Enabled:=FALSE; HW_TYPE_5.Enabled:=FALSE;
 Form1.Connection.down:=TRUE;
end;

{**********}

procedure TForm1.ServerSocket1ClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 {This message specifies that the client is connected to the server}
 Memo_Log.Lines.Add(Date_Str+': '+'Connection accepted by the server socket');
 CLIENT_CONNECTED:=TRUE;
 HW_TYPE_3.Enabled:=FALSE; HW_TYPE_4.Enabled:=FALSE; HW_TYPE_5.Enabled:=FALSE;
end;

{**********}

procedure TForm1.ServerSocket1ClientDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 {This message specifies that the client is disconnected from the server}
 Memo_Log.Lines.Add(Date_Str+': '+'Client disconnected from the server');
 CLIENT_CONNECTED:=FALSE;
 HW_TYPE_3.Enabled:=TRUE; HW_TYPE_4.Enabled:=TRUE; HW_TYPE_5.Enabled:=TRUE; 
// Form1.Disconnection.down:=TRUE;//However, another client can connect
 {WRITELN('Disconnect');}
  INITIALIZATION_AND_CONNECTION;
end;

{**********}

procedure TForm1.ServerSocket1ClientError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
var ErrorMsg: string;
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 STR(ErrorCode,ErrorMsg);
 Memo_Log.Lines.Add(Date_Str+'  Error: '+ErrorMsg+' Client disconnected from the server');
 {The error is managed}
 ErrorCode := 0;
 DateTime := Time; Date_str := TimeToStr(DateTime);
// Form1.Disconnection.down:=TRUE;//However, another client can connect
 CLIENT_CONNECTED:=FALSE;
 HW_TYPE_3.Enabled:=TRUE; HW_TYPE_4.Enabled:=TRUE; HW_TYPE_5.Enabled:=TRUE;
 Disconnection_TCP_IP;{of the client}
{WRITELN('Error');}
 INITIALIZATION_AND_CONNECTION;
end;

{**********}

PROCEDURE ANALYSE_Pversion_COMMAND_WITH_DATA_RECEIVED(LENGTH_RECEIVED:LONGINT;VAR ERRONEOUS_COMMAND:BOOLEAN);
VAR COUNTER,LENGTH_OF_THE_PARAMETER:LONGINT;
{The parameters must be entirely received before doing the analysis}
BEGIN
 {initialization}
 ERRONEOUS_COMMAND:=FALSE;

 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 LENGTH_OF_THE_PARAMETER:=ORD(CURRENT_STRING_RECEIVED[1]);
 {Note: an erroneous error can only come from the programmation, as TCP/IP don't made any errors}
 IF LENGTH_OF_THE_PARAMETER=0 THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;{at least one character}

 {"0" must be found as the first character}
 IF CURRENT_STRING_RECEIVED[2]<>'0' THEN BEGIN ERRONEOUS_COMMAND:=TRUE; EXIT; END;

 {Are there sufficiently characters, if yes: immediate analysis is done, if not: it will be expected the next data}
 IF LENGTH_RECEIVED>=(1+LENGTH_OF_THE_PARAMETER) THEN {Yes enough}
 BEGIN
  {This server only manages the version 0 of the protocol}
  CP_VERSION:=CURRENT_STRING_RECEIVED[2];
  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,1+LENGTH_OF_THE_PARAMETER);
  {Reinitialization}
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
  {The answer is PversionCHR(0), as in the server is only able of Version 0 (no choice to do)}
  {in other words, on reception on CHR(1)'0' (the sole version of Multipsk 4.31.4 is the 0), the server answers Version 0}
  TRANSMISSION_TCP_IP_COMMAND('Pversion'+CHR(0),TRUE);
  {New command to be received}
  COMMAND_TO_BE_RECEIVED:='HWMPcapa';{the second one to be received after connection}
  Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for 'HWMPcapa'}
 END;
END;

{**********}

PROCEDURE ANALYSE_HWMPCapa_COMMAND_WITH_DATA_RECEIVED(VAR ERRONEOUS_COMMAND:BOOLEAN);
VAR ANSWER:STRING;
VAR name,model,The_message,Types_managed_by_the_MP,Sampling_rates_managed_by_the_MP,LO_Frequencies_managed_by_the_MP:SHORTSTRING;
VAR GUI_managed_by_the_MP:BYTE;
{The parameters must be entirely received before doing the analysis}
BEGIN
 {initialization}
 ERRONEOUS_COMMAND:=FALSE;

 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 {No analysis, it is a direct request}
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';

 {The answer is HWMPcapa (name:SHORTSTRING; model:SHORTSTRING;
  Message:SHORTSTRING; Types_managed_by_the_MP:SHORTSTRING;
  Sampling_rates_managed_by_the_MP:SHORTSTRING;
  LO_Frequencies_managed_by_the_MP:SHORTSTRING;
  GUI_managed_by_the_MP:BYTE; Message:SHORTSTRING)}
 name:=BUILD_A_SHORTSTRING('Test SDR TCP/IP');
 model:=BUILD_A_SHORTSTRING('Server Version 0');
 The_message:=BUILD_A_SHORTSTRING('Hello');
 Types_managed_by_the_MP:=BUILD_A_SHORTSTRING(CHR(48+HARDWARE_TYPE)); {either '3' or '4' or '5'}
 Sampling_rates_managed_by_the_MP:=BUILD_A_SHORTSTRING('2000..48000');{Hz}
 LO_Frequencies_managed_by_the_MP:=BUILD_A_SHORTSTRING('10000..40000/50000..600000');{KHz}//Examples
 GUI_managed_by_the_MP:=0;{No GUI available}
 ANSWER:='HWMPcapa'+name+model+The_message+Types_managed_by_the_MP+Sampling_rates_managed_by_the_MP+LO_Frequencies_managed_by_the_MP;
 ANSWER:=ANSWER+CHR(GUI_managed_by_the_MP);
 {WRITELN('HWMPcapa ',ANSWER);}
 TRANSMISSION_TCP_IP_COMMAND(ANSWER,TRUE);
 {New command to be received}
 COMMAND_TO_BE_RECEIVED:='InitHWMP';{the third one to be received after connection}
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for 'InitHWMP'}
END;

{**********}

PROCEDURE ANALYSE_InitHWMP_COMMAND_WITH_DATA_RECEIVED(VAR ERRONEOUS_COMMAND:BOOLEAN);
VAR COUNTER,LENGTH_OF_THE_PARAMETER:LONGINT;
VAR type_selected,GUI_displayed:BYTE;
VAR Initial_sampling_rate_selected,Initial_LO_frequency_selected:LONGWORD;
VAR STRING1,STRING2:STRING;
BEGIN
 {initialization}
 ERRONEOUS_COMMAND:=FALSE;

 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 HARDWARE_TYPE:=ORD(CURRENT_STRING_RECEIVED[1])-48;{in fact, it is returned the HARDWARE_TYPE sent (3, 4 or 5), as there was no choice proposed}
 STRING1:=COPY(CURRENT_STRING_RECEIVED,2,4);
 Initial_sampling_rate_selected:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(STRING1);
 STRING2:=COPY(CURRENT_STRING_RECEIVED,6,4);
 Initial_LO_frequency_selected:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(STRING2);
 STRING1:=COPY(CURRENT_STRING_RECEIVED,10,1);
 GUI_displayed:=ORD(STRING1[1]);{0 will be received as this server cannot display even a simulated GUI}
 {here is normally located a possible checking of these parameters with, perhaps, finally an ERRONEOUS_COMMAND
  and then the management of the interface with the hardware with these parameters}
 {..................................................................................................}

 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,10);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 {The initialisation is now finished. The HW/MP must only used TXHWdata (or HW_Pause or Close_HW)}
 COMMAND_TO_BE_RECEIVED:='';{no precise command to receive. It could be TXHWdata or HW_Pause or Bug_on_P}

 {The first thing to do, for an hardware Type 3, is to send available data as soon as data are ready
 For the hardware Type 4, it is regularly (each second) sent an empty TXHWdata to be sure that the link is effective}
 DATA_MUST_BE_SENT:=TRUE;
 IF HARDWARE_TYPE=4 THEN Initial_time_of_TXHWdata_transmission:=GetTickCount;

 {In this test server there is no HW so pseudo I and Q data will be generated in the timer (the timer playing the Hardware)}
END;

{**********}

PROCEDURE ANALYSE_Bug_on_P_COMMAND_WITH_DATA_RECEIVED(LENGTH_RECEIVED:LONGINT);
VAR COUNTER,LENGTH_OF_THE_PARAMETER:LONGINT;
VAR TEXT_RECEIVED:STRING;
{The parameters must be entirely received before doing the analysis}
BEGIN
 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 LENGTH_OF_THE_PARAMETER:=ORD(CURRENT_STRING_RECEIVED[1]);

 {Are there sufficiently characters, if yes: immediate analysis is done, if not: it will be expected the next data}
 IF LENGTH_RECEIVED>=(1+LENGTH_OF_THE_PARAMETER) THEN {Yes enough}
 BEGIN
  TEXT_RECEIVED:=COPY(CURRENT_STRING_RECEIVED,2,LENGTH_OF_THE_PARAMETER);
  Form1.Label_Info.Caption:=TEXT_RECEIVED;{Bug information displayed}

  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,1+LENGTH_OF_THE_PARAMETER);
  {Reinitialization}
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
  Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
 END;
END;

{**********}

PROCEDURE ANALYSE_Received_COMMAND;
{Just to express presence of the MP (and so to reintizialize the time counter}
BEGIN
 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_SwitchTX_COMMAND;
{V.1}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 {Here the server switches the Hardware in TX position, ready to receive data from the client program, for transmission}
 {..................................................................................................}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 RECEPTION_DATA_IN_PROGRESS:=TRUE; {Client in TX}
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_SwitchRX_COMMAND;
{V.1}
BEGIN
 {deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 {Here the server switches the Hardware in RX position, ready to tranmsmit data to the client program}
 {..................................................................................................}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 RECEPTION_DATA_IN_PROGRESS:=FALSE; {Client in RX}
 {The first thing to do, for an hardware Type 3, is to send available data as soon as data are ready}
 DATA_MUST_BE_SENT:=TRUE;
 INITIAL_TIME_OF_DATA_TRANSMISSION:=0;
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
END;

{**********}

PROCEDURE ANALYSE_TXHWdata_COMMAND_WITH_DATA_RECEIVED(LENGTH_OF_THE_PARAMETER:DWORD);
{V.1 Reception of data}
VAR DATA_RECEIVED,IQ:STRING;
VAR COUNTER:DWORD;
VAR I,Q:SMALLINT;

 {!!!!!!!!}

 PROCEDURE TRANSFORMATION_OF_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_I_AND_Q(STRING1:STRING;VAR I:SMALLINT;VAR Q:SMALLINT);
 {Extraction of I and Q from a string of four characters
 It is substracted 32767 as expected by the protocol so to have I and Q between -32767 and 32767}
 //VAR BYTE1,BYTE2,BYTE3,BYTE4:LONGINT;
 VAR I_2c,Q_2c:WORD;{received in 2'complement}
 BEGIN
//  BYTE1:=ORD(THE_LONGWORD[1]); {weakest byte of I}
//  BYTE2:=ORD(THE_LONGWORD[2]); {strongest byte of I}
//  BYTE3:=ORD(THE_LONGWORD[3]); {weakest byte of Q}
//  BYTE4:=ORD(THE_LONGWORD[4]); {strongest byte of Q}
  I_2c:=ORD(STRING1[2])*256+ORD(STRING1[1]);
  Q_2c:=ORD(STRING1[4])*256+ORD(STRING1[3]);
  I:=I_2c; Q:=Q_2c;{2's complement number to signed number (from -32767 to 32767}
 END;

BEGIN
 {deletion of the command (8 characters)+the lenght on 4 bytes}
 DELETE(CURRENT_STRING_RECEIVED,1,12);

 {extraction of data}
 DATA_RECEIVED:=COPY(CURRENT_STRING_RECEIVED,1,LENGTH_OF_THE_PARAMETER);
 IF LENGTH_OF_THE_PARAMETER>0 THEN {LENGTH_OF_THE_PARAMETER=0 in case of empty TXHWdata (for a Hardware type 4)}
 BEGIN
  {this loop is optional depending on the way the user wishes to manage received data.
  It is reminded that the data is received in format 16 bits little endian, 2'complement}
  FOR COUNTER:=0 TO (LENGTH_OF_THE_PARAMETER DIV 4)-1 DO {LENGTH_OF_THE_PARAMETER DIV 4: number of parameters (IQ with 16 bits for I and Q}
  BEGIN
   IQ:=COPY(DATA_RECEIVED,COUNTER*4+1,4);
   TRANSFORMATION_OF_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_I_AND_Q(IQ,I,Q);
   {...................................................................}
  END;
  {End of analysis, updating of the strings with the part analysed}
  DELETE(CURRENT_STRING_RECEIVED,1,LENGTH_OF_THE_PARAMETER);
 END;

 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of waiting for a new procedure}
 {Display useless after having being correctly connected and controlled (as it takes CPU)}
 TRANSMISSION_TCP_IP_COMMAND('Received',FALSE);
END;

{**********}

PROCEDURE ANALYSE_Set_HWLO_Command(VAR ERRONEOUS_COMMAND:BOOLEAN);
{New LO frequency}
VAR LO_frequency:DWORD;
VAR FREQUENCY,STRING1:STRING;
BEGIN
 {initialization}
 ERRONEOUS_COMMAND:=FALSE;

 {V.1 deletion of the command (8 characters)}
 DELETE(CURRENT_STRING_RECEIVED,1,8);

 STRING1:=COPY(CURRENT_STRING_RECEIVED,1,4);
 LO_frequency:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(STRING1);
 {V.0.1}
 STR(LO_frequency,FREQUENCY);
 Form1.Memo_Reception_Text.Lines.Add('RX frequency (Hz): '+FREQUENCY);
 {here is normally located a possible checking of this parameter with, perhaps, finally an ERRONEOUS_COMMAND
  and then the management of the interface with the hardware with this new LO frequency}
 {..................................................................................................}

 {End of analysis, updating of the strings with the part analysed}
 DELETE(CURRENT_STRING_RECEIVED,1,4);
 {Reinitialization}
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE; COMMAND_RECEIVED:='';
 {The initialisation is now finished. The HW/MP must only used TXHWdata (or HW_Pause or Close_HW)}
 COMMAND_TO_BE_RECEIVED:='';{no precise command to receive. It could be TXHWdata or HW_Pause or Bug_on_P}
END;

{**********}

PROCEDURE INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION:STRING);
{V.1}
VAR LENGTH_INFORMATION:LONGINT;
BEGIN
 LENGTH_INFORMATION:=LENGTH(BUG_INFORMATION);
 IF LENGTH_INFORMATION>255 THEN
 BEGIN
  BUG_INFORMATION:=COPY(BUG_INFORMATION,1,255);
  LENGTH_INFORMATION:=255;
 END;
 Non_blocking_Delay(10);{a small delay by precaution}
 TRANSMISSION_TCP_IP_COMMAND('Bug_on_P'+CHR(LENGTH_INFORMATION)+BUG_INFORMATION,TRUE);
 Non_blocking_Delay(100);{Leave the message be transmitted}
{WRITELN('Bug_on_P');}
 Disconnection_TCP_IP;
 INITIALIZATION_AND_CONNECTION;
END;

{**********}

procedure TForm1.ServerSocket1ClientRead(Sender: TObject; Socket: TCustomWinSocket);
VAR COMMAND,RECEIVED,RECEIVED_IN_CAPITALS,STRING1:STRING;
VAR INITIAL_LENGTH{V.1},LENGTH_RECEIVED:LONGINT;
VAR ERRONEOUS_COMMAND,COMMAND_COMPLETELY_ANALYSED:BOOLEAN;
VAR BUG_INFORMATION:STRING;
VAR LENGTH_INFORMATION:LONGINT;
VAR INSUFFICIENT_LENGTH_RECEIVED:BOOLEAN;
VAR LENGTH_OF_THE_PARAMETER:DWORD;
VAR COMPTEUR:DWORD;{V.1}

 {!!!!!!!!}
 PROCEDURE UPDATING_FOLLOWING_COMMAND_RECEPTION(COUNTER:LONGINT);
 BEGIN
  TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=TRUE;
  COMMAND_RECEIVED:=COMMAND;
  {V.0.1 updating of the string received, by deleting the part before the extracted command}
  DELETE(CURRENT_STRING_RECEIVED,1,COUNTER-1);
  LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 END;

 {!!!!!!!!}
 PROCEDURE MANAGEMENT_PART_OF_THE_TCP_IP_RECEPTION;
 VAR COUNTER:LONGINT;
 BEGIN
  COMMAND:='';{initialization}
  INSUFFICIENT_LENGTH_RECEIVED:=FALSE;

  {Analysis of the characters received to detect a command}
  IF TCP_IP_COMMAND_RECEPTION_IN_PROGRESS=FALSE THEN {New command}
  BEGIN
   FOR COUNTER:=1 TO LENGTH_RECEIVED-7 DO {-7 because the length of all commands is 8 (and the first character is number 1, not 0)}
   BEGIN
    COMMAND:=COPY(CURRENT_STRING_RECEIVED,COUNTER,8);
    IF (COMMAND='Pversion') AND (COMMAND_TO_BE_RECEIVED='Pversion') THEN
    BEGIN
     Initial_time_of_an_event_for_waiting_delay:=GetTickCount;{the event is the beginning of reception of a command}
     CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=TRUE;{now, after reception of the command Pversion, the control can begin}
     UPDATING_FOLLOWING_COMMAND_RECEPTION(COUNTER);
     BREAK;
    END ELSE
    IF ((COMMAND='HWMPcapa') AND (COMMAND_TO_BE_RECEIVED='HWMPcapa')) OR
       ((COMMAND='InitHWMP') AND (COMMAND_TO_BE_RECEIVED='InitHWMP')) OR
       (COMMAND='Bug_on_P') OR (COMMAND='Received') OR (COMMAND='Set_HWLO') OR
       (COMMAND='SwitchTX') OR (COMMAND='SwitchRX') OR (COMMAND='TXHWdata') {V.1} THEN
    BEGIN
     UPDATING_FOLLOWING_COMMAND_RECEPTION(COUNTER);
     BREAK;
    END;
   END;
  END;
  IF TCP_IP_COMMAND_RECEPTION_IN_PROGRESS=FALSE THEN {failure to find a command}
  BEGIN
   {updating of the strings with the part analysed}
   DELETE(CURRENT_STRING_RECEIVED,1,LENGTH_RECEIVED-7);
   EXIT;
  END ELSE {a command has been received, which must be managed}
  BEGIN
   ERRONEOUS_COMMAND:=FALSE;{initialization}

   {the minimum length for parameters for Pversion is two characters}
   IF (COMMAND_RECEIVED='Pversion') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+2) THEN ANALYSE_Pversion_COMMAND_WITH_DATA_RECEIVED(LENGTH_RECEIVED,ERRONEOUS_COMMAND)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   IF (COMMAND_RECEIVED='HWMPcapa') THEN ANALYSE_HWMPCapa_COMMAND_WITH_DATA_RECEIVED(ERRONEOUS_COMMAND) ELSE

   {the parameters length for InitHWMP is 10 characters}
   IF (COMMAND_RECEIVED='InitHWMP') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+10) THEN ANALYSE_InitHWMP_COMMAND_WITH_DATA_RECEIVED(ERRONEOUS_COMMAND)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   {the minimum length for parameters for Bug_on_P_command is one character}
   IF (COMMAND_RECEIVED='Bug_on_P') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+1) THEN ANALYSE_Bug_on_P_COMMAND_WITH_DATA_RECEIVED(LENGTH_RECEIVED)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   IF (COMMAND_RECEIVED='Received') THEN ANALYSE_Received_COMMAND ELSE

   {the parameter length for Set_HWLO is 4 characters}
   IF (COMMAND_RECEIVED='Set_HWLO') THEN
   BEGIN
    IF (LENGTH_RECEIVED>=8+4)  THEN ANALYSE_Set_HWLO_Command(ERRONEOUS_COMMAND)
    ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END ELSE

   IF (COMMAND_RECEIVED='SwitchTX') THEN ANALYSE_SwitchTX_COMMAND ELSE{V.1}

   IF (COMMAND_RECEIVED='SwitchRX') THEN ANALYSE_SwitchRX_COMMAND ELSE{V.1}

   {the length for parameter for TXHWdata must be determined first. To received the length parameter, 4 characters are necessary}
   IF (COMMAND_RECEIVED='TXHWdata') THEN{V.1}
   BEGIN
    IF (LENGTH_RECEIVED>=8+4) THEN
    BEGIN
     LENGTH_OF_THE_PARAMETER:=TRANSFORMATION_FOUR_CHARACTERS_LITTLE_ENDIAN_IN_LONGWORD(COPY(CURRENT_STRING_RECEIVED,8+1,4));
     {WRITELN('LL ',LENGTH_RECEIVED,' ',LENGTH_OF_THE_PARAMETER+8+4);}
     IF LENGTH_RECEIVED>=(LENGTH_OF_THE_PARAMETER+8+4) THEN ANALYSE_TXHWdata_COMMAND_WITH_DATA_RECEIVED(LENGTH_OF_THE_PARAMETER)
     ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
    END ELSE INSUFFICIENT_LENGTH_RECEIVED:=TRUE;
   END;

   IF ERRONEOUS_COMMAND=TRUE THEN
   BEGIN
    {The client is warned, by a bug information, before being disconnected}
    BUG_INFORMATION:='Incorrect command: '+COMMAND+CURRENT_STRING_RECEIVED;
    INFORMATION_AND_DISCONNECTION_ON_A_BUG(BUG_INFORMATION);{V.1}
   END;
  END;
 END;

begin
 {V.1 same way to receive data as the client}

 {updating}
 RECEIVED:=Socket.ReceiveText;

 {WRITELN(COPY(RECEIVED,5,8));}
 {V.0.1, The 4 four first characters are the lenght of the command: this length is received but is ignored here, but could be used by programs, as it can simply reception)}
 {V.1 Display except "Received" and TXHWdata'}
 IF (COPY(RECEIVED,5{V.0.1},8)<>'TXHWdata') AND (COPY(RECEIVED,5{V.0.1},8)<>'Received') THEN Form1.Memo_reception_Text.Lines.Add(Date_str+': '+COPY(RECEIVED,5,20));

  {**********here begin the short saving and display part*********}
 {V.1 Display of the Kdata received}
 INC(NUMBER_OF_CHARACTERS_RECEIVED,LENGTH(RECEIVED));
 COMPTEUR:=NUMBER_OF_CHARACTERS_RECEIVED DIV 1000;
 IF COMPTEUR>PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED THEN
 BEGIN
  STR(COMPTEUR,STRING1);
  NOMBRE_MILLIERS_RX.Caption:='N RX(Kdata)='+STRING1;
  PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED:=COMPTEUR;//rinitialisation
 END;

 {Updating of STRING_RECEIVED_AND_SAVED}
 STRING_RECEIVED_AND_SAVED:=STRING_RECEIVED_AND_SAVED+RECEIVED;

 IF MANAGEMENT_TCP_IP_RECEPTION_FINISHED=FALSE THEN EXIT; {it is necessary to have finished the process before to re-begin a new one}
 {**********here begin the management part*********}
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=FALSE;

  {We must work with a "private" variable to be independant of STRING_RECEIVED_AND_SAVED
  in case of multiple call of this procedure ClientSocket1Read}
 CURRENT_STRING_RECEIVED:=STRING_RECEIVED_AND_SAVED;
 LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 INITIAL_LENGTH:=LENGTH_RECEIVED;
 IF LENGTH_RECEIVED<8 THEN BEGIN MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE; EXIT; END;{minimum for a single command: 8 characters}
 REPEAT
  INSUFFICIENT_LENGTH_RECEIVED:=FALSE;
  MANAGEMENT_PART_OF_THE_TCP_IP_RECEPTION;
  {updating}
  LENGTH_RECEIVED:=LENGTH(CURRENT_STRING_RECEIVED);
 UNTIL (LENGTH_RECEIVED<8) OR (INSUFFICIENT_LENGTH_RECEIVED=TRUE); {minimum for a single command: 8 characters}
 {management of the STRING_RECEIVED_AND_SAVED string according to the number of bytes managed}
 IF INITIAL_LENGTH-LENGTH_RECEIVED>0 THEN DELETE(STRING_RECEIVED_AND_SAVED,1,INITIAL_LENGTH-LENGTH_RECEIVED);
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE;
end;

{**********}

procedure TForm1.ServerSocket1Listen(Sender: TObject;
  Socket: TCustomWinSocket);
begin
 DateTime := Time; Date_str := TimeToStr(DateTime);
 Memo_Log.Lines.Add(Date_Str+': '+'Listening...');
end;

{***********}

procedure TForm1.Manual_commandClick(Sender: TObject);
VAR COMMAND:STRING;
begin
 COMMAND:=Edit1.Text;
 TRANSMISSION_TCP_IP_COMMAND(COMMAND,TRUE);
end;

{*********}

procedure TForm1.HW_TYPE_3Click(Sender: TObject);
begin
 HARDWARE_TYPE:=3;
end;

{*********}

procedure TForm1.HW_TYPE_4Click(Sender: TObject);
begin
 HARDWARE_TYPE:=4;
end;

{*********}

procedure TForm1.HW_TYPE_5Click(Sender: TObject);
begin
 HARDWARE_TYPE:=5;
end;

{*********}

procedure INITIALIZATION_WITHOUT_CONNECTION;
{Following a HW problem, the  server has been stopped, but prepare to connect}
begin
 {Initialization}
 Form1.Memo_reception_Text.setfocus;
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE;
 COMMAND_RECEIVED:='';
 COMMAND_TO_BE_RECEIVED:='Pversion';{the first one to be received after connection}
 CURRENT_STRING_RECEIVED:='';
 Initial_time_of_an_event_for_waiting_delay:=0;
 CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;
 CLIENT_CONNECTED:=FALSE;
 FORM1.HW_TYPE_3.Enabled:=TRUE; FORM1.HW_TYPE_4.Enabled:=TRUE; FORM1.HW_TYPE_5.Enabled:=TRUE;
 SERVER_IN_SERVICE:=FALSE;
 {simulations configuration}
 INITIAL_TIME_OF_DATA_TRANSMISSION:=0; DATA_MUST_BE_SENT:=FALSE; SIMULATED_DATA_TO_SEND:=TRUE; FORM1.TX_Data.down:=TRUE;
 FORM1.Pause.down:=FALSE; FORM1.HW_problem.down:=FALSE;
 {Disconnection configured}
 Form1.Disconnection.Down:=TRUE;
 Form1.Connection.Down:=FALSE;
end;

{*********}

procedure INITIALIZATION_AND_CONNECTION;
begin
{WRITELN('Init');}
 {Initialization}
 Form1.Memo_reception_Text.setfocus;
 TCP_IP_COMMAND_RECEPTION_IN_PROGRESS:=FALSE;
 COMMAND_RECEIVED:='';
 COMMAND_TO_BE_RECEIVED:='Pversion';{the first one to be received after connection}
 CURRENT_STRING_RECEIVED:='';
 STRING_RECEIVED_AND_SAVED:='';//V.1
 Initial_time_of_an_event_for_waiting_delay:=0;
 CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;
 MANAGEMENT_TCP_IP_RECEPTION_FINISHED:=TRUE;//V.1
 CLIENT_CONNECTED:=FALSE;
 FORM1.HW_TYPE_3.Enabled:=TRUE; FORM1.HW_TYPE_4.Enabled:=TRUE; FORM1.HW_TYPE_5.Enabled:=TRUE;
 SERVER_IN_SERVICE:=FALSE;
 NUMBER_OF_CHARACTERS_RECEIVED:=0;
 PREVIOUS_NUMBER_OF_CHARACTERS_RECEIVED:=0;
 PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED:=0;
 NUMBER_OF_CHARACTERS_TRANSMITTED:=0;
 RECEPTION_DATA_IN_PROGRESS:=FALSE; {Client in RX initially}
 {simulations configuration}
 INITIAL_TIME_OF_DATA_TRANSMISSION:=0; DATA_MUST_BE_SENT:=FALSE; SIMULATED_DATA_TO_SEND:=TRUE; FORM1.TX_Data.down:=TRUE;
 FORM1.Pause.down:=FALSE; FORM1.HW_problem.down:=FALSE;
 {Immediate connection}
 Form1.Connection.Down:=TRUE;
 Connection_TCP_IP;
end;

{*******}

procedure TForm1.Timer1Timer(Sender: TObject);
{Paquets are sent by 8092 characters (2048 IQ samples). So the timer1 frequency must superior to
48000/2048=23 (to receive a sole paquet, even if not critical).
Set it to the fastest possible (interval=1) is the good solution (in reality the maximum is around 80/sec)}
VAR BUG_INFORMATION:STRING;
VAR LENGTH_INFORMATION:WORD;
VAR DELTA_TIME,NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT:DWORD;
VAR NUMBER_OF_IQ_SAMPLES_TO_SEND:DWORD;
VAR NUMBER_OF_PERIODS_OF_SIGNAL_TO_SEND,COUNTER:DWORD;
VAR STRING_TO_SEND,STRING0:STRING;
VAR COMPTEUR:DWORD;
VAR Min,Max:SMALLINT;
VAR Final_min,Final_max:WORD;{signed number to 2's complement number}
VAR STRING1:STRING;

 {!!!!!!!!}

 PROCEDURE TRANSMISSION_OF_I_Q(I,Q:WORD);
 VAR STRING1:STRING;
 BEGIN
  STRING1:=CHR(I MOD 256)+CHR(I DIV 256)+CHR(Q MOD 256)+CHR(Q DIV 256);
  STRING_TO_SEND:=STRING_TO_SEND+STRING1;
 END;

begin
 {The MP server, must consider that after 15 seconds waiting for a CP event the CP has some problem and the MP will disconnect the CP.}
 IF (CONTROL_OF_MAXIMUM_DELAY_ACTIVATED=TRUE) AND ((GetTickCount-Initial_time_of_an_event_for_waiting_delay)>Maximum_delay_to_receive_the_expected_event) THEN
 BEGIN
  CONTROL_OF_MAXIMUM_DELAY_ACTIVATED:=FALSE;{to avoid re-coming}
  DateTime := Time; Date_str := TimeToStr(DateTime);
  Memo_Log.Lines.Add(Date_Str+': '+'Client absent from more than 15 seconds, so automatic disconnection');
  {The client is warned, by a bug information before being disconnected}
  BUG_INFORMATION:='Disconnection due to a too long waiting time';
  LENGTH_INFORMATION:=LENGTH(BUG_INFORMATION);
  Non_blocking_Delay(10);{a small delay is indispensable}
  TRANSMISSION_TCP_IP_COMMAND('Bug_on_P'+CHR(LENGTH_INFORMATION)+BUG_INFORMATION,TRUE);
  Non_blocking_Delay(100);{Leave the message be transmitted}
  Disconnection_TCP_IP;{of the client}
  INITIALIZATION_AND_CONNECTION;
 END;

 {For the hardware Type 4, it is regularly (each second) sent an empty TXHWdata to be sure that the link is effective}
 IF (HARDWARE_TYPE=4) AND (DATA_MUST_BE_SENT=TRUE) AND (GetTickCount-Initial_time_of_TXHWdata_transmission>=1000) THEN
 BEGIN
  {empty TXHWdata}
  TRANSMISSION_TCP_IP_COMMAND('TXHWdata'+CHR(0)+CHR(0)+CHR(0)+CHR(0),FALSE);
  Initial_time_of_TXHWdata_transmission:=GetTickCount;{reinitialization}
  {To show activity}
  IF NOMBRE_MILLIERS_TX.Caption='.' THEN NOMBRE_MILLIERS_TX.Caption:='..' ELSE
  IF NOMBRE_MILLIERS_TX.Caption='..' THEN NOMBRE_MILLIERS_TX.Caption:='...' ELSE
  IF NOMBRE_MILLIERS_TX.Caption='...' THEN NOMBRE_MILLIERS_TX.Caption:='....' ELSE
  IF NOMBRE_MILLIERS_TX.Caption='....' THEN NOMBRE_MILLIERS_TX.Caption:='.....' ELSE NOMBRE_MILLIERS_TX.Caption:='.';
 END;

 {Data transmission in Hardware type 3 or 5 and Client en RX}
 {It is important to not re-enter in the procudure before the end, so use of Timer1.Enabled}
 IF (HARDWARE_TYPE<>4) AND (RECEPTION_DATA_IN_PROGRESS=FALSE) AND (DATA_MUST_BE_SENT=TRUE) AND (SIMULATED_DATA_TO_SEND=TRUE) THEN {Simulation of data transmitted}
 {It is simulated a square signal at a frequency of 6000 Hz with a SR (sampling frequency) of 48000 Hz and 16 bits for I and Q.
 So the period of the signal will equal to 8 samples SR=48000/6000.
 With a period of 8 samples the Q is out of phase of 2 samples (Pi/2) from I.
 However this permits to have two traces on Multipsk, on each side of the SdR waterfall
 The timer must be the most quick possible to send packets of 3000 bytes or less (consider 4000 byts as a maximum).
 Above, the CPU load increases up to possibly freezes}
 BEGIN
  {It is important to not re-enter in the procudure before the end, so use of Timer1.Enabled}
  Timer1.Enabled:=FALSE;
  {First call to this function, initialization}
  IF INITIAL_TIME_OF_DATA_TRANSMISSION=0 THEN
  BEGIN
   INITIAL_TIME_OF_DATA_TRANSMISSION:=GetTickCount;
   NUMBER_OF_IQ_SAMPLES_SENT:=0;
  END ELSE
  {Next calls}
  BEGIN
   DELTA_TIME:=GetTickCount-INITIAL_TIME_OF_DATA_TRANSMISSION;
   NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT:=48*DELTA_TIME;{normally: (48000*DELTA_TIME) DIV 1000};
   NUMBER_OF_IQ_SAMPLES_TO_SEND:=NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT-NUMBER_OF_IQ_SAMPLES_SENT;
   IF NUMBER_OF_IQ_SAMPLES_TO_SEND=0 THEN EXIT;
   {Transmission with Q delayed from I of Pi/2}
   {NUMBER_OF_IQ_SAMPLES_TO_SEND DIV 8 is the number of periods (with 8 samples/period at 6000 Hz}
   STRING_TO_SEND:='';

   {The initial signed signal is symmetrical between -32767  32767. It is produced here a square signal
   so between min and max}
   Min:=-32767; Max:=32767;
   Final_min:=Min;    Final_max:=Max; {signed number to 2's complement number: SMALLINIT to WORD}
   FOR COUNTER:=1 TO (NUMBER_OF_IQ_SAMPLES_TO_SEND DIV 8) DO
   BEGIN
    {for test: TRANSMISSION_OF_I_Q(12336,12336); TRANSMISSION_OF_I_Q(12593,12593); TRANSMISSION_OF_I_Q(12850,12850); TRANSMISSION_OF_I_Q(13107,13107);
    TRANSMISSION_OF_I_Q(13364,13364); TRANSMISSION_OF_I_Q(13621,13621); TRANSMISSION_OF_I_Q(13878,13878); TRANSMISSION_OF_I_Q(14135,14135);}
    {Sqaure wave signal}
    TRANSMISSION_OF_I_Q(Final_min,Final_max);
    TRANSMISSION_OF_I_Q(Final_min,Final_max);
    TRANSMISSION_OF_I_Q(Final_min,Final_min);
    TRANSMISSION_OF_I_Q(Final_min,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_min);
    TRANSMISSION_OF_I_Q(Final_max,Final_max);
    TRANSMISSION_OF_I_Q(Final_max,Final_max);
   END;
   {16 bits for I and Q, so 32 bits for I+Q or 4 characters}
   STRING0:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(NUMBER_OF_IQ_SAMPLES_TO_SEND*4);
   {Display useless after having being correctly connected and controlled (as it takes CPU)}
   IF (CLIENT_CONNECTED=TRUE) AND (SERVER_IN_SERVICE=TRUE) THEN{in case of disconnection while in the timer}
   BEGIN
    TRANSMISSION_TCP_IP_COMMAND('TXHWdata'+STRING0+STRING_TO_SEND,FALSE);
    INC(NUMBER_OF_IQ_SAMPLES_SENT,NUMBER_OF_IQ_SAMPLES_TO_SEND);

    {for test: STR(NUMBER_OF_IQ_SAMPLES_TO_SEND,STRING0); Button1.Caption;}

    {Display of the Kdata transmitted}
    INC(NUMBER_OF_CHARACTERS_TRANSMITTED,NUMBER_OF_IQ_SAMPLES_TO_SEND*4);
    COMPTEUR:=NUMBER_OF_CHARACTERS_TRANSMITTED DIV 1000;
    IF COMPTEUR>PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED THEN
    BEGIN
     STR(COMPTEUR,STRING0);
     NOMBRE_MILLIERS_TX.Caption:='N TX(Kdata)='+STRING0;
     PREVIOUS_NUMBER_OF_CHARACTERS_TRANSMITTED:=COMPTEUR;//rinitialisation
    END;

    {To avoid to pass the DWORD length}
    IF NUMBER_OF_IQ_SAMPLES_WHICH_MUST_HAVE_NORMALLY_BEEN_SENT>480000000 THEN
    BEGIN
     INC(INITIAL_TIME_OF_DATA_TRANSMISSION,10000000);
     DEC(NUMBER_OF_IQ_SAMPLES_SENT,480000000);
    END;
   END;
  END;
  Timer1.Enabled:=TRUE;{Timer ready now that it is finished}
 END;
end;

{*******}

procedure TForm1.HW_problemClick(Sender: TObject);
VAR BUG_INFORMATION:STRING;
VAR LENGTH_INFORMATION:WORD;
begin
  DateTime := Time; Date_str := TimeToStr(DateTime);
  Memo_Log.Lines.Add(Date_Str+': '+'Hardware problem, so client disconnection');
  {The client is warned, by a bug information before being disconnected}
  BUG_INFORMATION:='Sorry, disconnection due to an hardware problem';
  LENGTH_INFORMATION:=LENGTH(BUG_INFORMATION);
  Non_blocking_Delay(10);{a small delay is a precaution}
  TRANSMISSION_TCP_IP_COMMAND('Bug_on_P'+CHR(LENGTH_INFORMATION)+BUG_INFORMATION,TRUE);
  Non_blocking_Delay(100);{Leave the message be transmitted}
  Disconnection_TCP_IP;{of the client}
  INITIALIZATION_WITHOUT_CONNECTION;
end;

{********}

procedure TForm1.PauseClick(Sender: TObject);
{Simulation of a pause of 3 seconds}
begin
 Non_blocking_Delay(10);{a small delay is a precaution}
 TRANSMISSION_TCP_IP_COMMAND('HW_Pause'+CHR(3),TRUE);
 Non_blocking_Delay(100);{Leave the message be transmitted}
 Sleep(3000);{Blocking waiting}
end;

{*********}

procedure TForm1.Adjust_rateClick(Sender: TObject);
{Simulation of a demand of flow data rate at 36000 samples/sec (instead of 48000) to reduce the data buffer}
begin
 Non_blocking_Delay(10);{a small delay is a precaution}
 TRANSMISSION_TCP_IP_COMMAND('ADJUrate'+CHR(40),TRUE);{40 is worth for 36000 sampls/sec}
 Non_blocking_Delay(100);{Leave the message be transmitted}
end;

{********}

procedure TForm1.FrequencyClick(Sender: TObject);
VAR FREQUENCY_HZ:DWORD;
VAR STRING0:STRING;
begin
 FREQUENCY_HZ:=123456789;
 STRING0:=TRANSFORMATION_LONGWORD_IN_CHARACTERS_LITTLE_ENDIAN(FREQUENCY_HZ);
 Non_blocking_Delay(10);{a small delay is a precaution}
 TRANSMISSION_TCP_IP_COMMAND('Set_HWLO'+STRING0,TRUE);
 Non_blocking_Delay(100);{Leave the message be transmitted}
end;

{*********}

procedure TForm1.TX_DataClick(Sender: TObject);
begin
 SIMULATED_DATA_TO_SEND:=TX_data.Down;
 INITIAL_TIME_OF_DATA_TRANSMISSION:=0;
end;

{*********}

procedure TForm1.FormActivate(Sender: TObject);
begin
 HARDWARE_TYPE:=3; HW_TYPE_3.Down:=TRUE;{by default}
 INITIALIZATION_AND_CONNECTION;
end;



end.
